unit mxmetafile;

(******************************************************************************
  mxmetafile.pas
    Module for working with metafiles
  
  Copyright notice:

 (c) 2008 Zolotov Alex

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Zolotov Alex, 2008
    zolotov-alex@shamangrad.net
    http://shamangrad.net/

 ******************************************************************************)
 
{$ifdef FPC}
  {$mode delphi}
  {$h+}
  {$assertions on}
{$endif}

interface
uses SysUtils, Classes, mxutils;

type
  TMXMetafile = class;
  TMXMetafileSection = class;
  
  TMXMetafileOption = class
    private
      FName: UTF8String;
      FValue: UTF8String;
      FComment: UTF8String;
      FSection: TMXMetafileSection;
      
      function GetMetafile: TMXMetafile;
    public
      constructor Create(ASection: TMXMetafileSection; const AName: UTF8String = '';
        const AValue: UTF8String = ''; const AComment: UTF8String = '');
      
      property Name: UTF8String read FName write FName;
      property Value: UTF8String read FValue write FValue;
      property Comment: UTF8String read FComment write FComment;
      property Section: TMXMetafileSection read FSection;
      property Metafile: TMXMetafile read GetMetafile;
  end;
  
  TMXMetafileSection = class
    private
      FName: UTF8String;
      FComment: UTF8String;
      FOptions: TList;
      FMetafile: TMXMetafile;
      
      function GetOptionCount: Integer;
      function GetOption(Idx: Integer): TMXMetafileOption;
    public
      constructor Create(AMetafile: TMXMetafile; const AName: UTF8String;
        AComment: UTF8String = '');
      destructor Destroy; override;
      
      procedure Clear;
      function IndexOfFirstOption(const Name: UTF8String): Integer;
      function GetFirstOption(const Name: UTF8String): TMXMetafileOption;
      function AddOption(const Option, Value: UTF8String): Integer; overload;
      function AddOption(const Option, Value: UTF8String; const Comment: UTF8String): Integer; overload;
      procedure DeleteOption(Idx: Integer);
      procedure RemoveOption(AOption: TMXMetafileOption);
      
      procedure SaveToFile(var AFile: TextFile);
      
      property Name: UTF8String read FName write FName;
      property Comment: UTF8String read FComment write FComment;
      property OptionCount: Integer read GetOptionCount;
      property Option[Idx: Integer]: TMXMetafileOption read GetOption;
      property Metafile: TMXMetafile read FMetafile;
  end;
  
  TMXMetafile = class
    private
      FSections: TStringList;
      
      function GetSectionCount: Integer;
      function GetSectionByIdx(Idx: Integer): TMXMetafileSection;
      function GetSectionByName(const Name: UTF8String): TMXMetafileSection;
      function GetSectionSize(const Section: UTF8String): Integer;
      function GetOption(const Section: UTF8String; OptionIdx: Integer): TMXMetafileOption;
    public
      constructor Create;
      destructor Destroy; override;
      
      procedure Clear;
      function IndexOfSection(const Section: UTF8String): Integer;
      function LookupSection(const Section: UTF8String): TMXMetafileSection;
      function LookupSectionIdx(const Section: UTF8String): Integer;
      procedure DeleteSection(Idx: Integer);
      procedure RemoveSection(const Section: UTF8String); overload;
      procedure RemoveSection(ASection: TMXMetafileSection); overload;
      procedure AddOption(const Section, Option, Value: UTF8String); overload;
      procedure AddOption(const Section, Option, Value, Comment: UTF8String); overload;
      procedure DeleteOption(const Section: UTF8String; Idx: Integer);
      procedure RemoveOption(AOption: TMXMetafileOption);
      
      procedure LoadFromFile(const FileName: string);
      procedure LoadFromStream(Stream: TStream);
      procedure SaveToFile(const FileName: string);
      procedure SaveToStream(Stream: TStream);
      
      property SectionCount: Integer read GetSectionCount;
      property Items[Idx: Integer]: TMXMetafileSection read GetSectionByIdx;
      property Section[const Name: UTF8String]: TMXMetafileSection read GetSectionByName;
      property SectionSize[const Name: UTF8String]: Integer read GetSectionSize;
      property Option[const Section: UTF8String; Idx: Integer]: TMXMetafileOption read GetOption;
  end;
  
implementation

constructor TMXMetafileOption.Create(ASection: TMXMetafileSection; const AName: UTF8String = '';
  const AValue: UTF8String = ''; const AComment: UTF8String = '');
begin
  FSection := ASection;
  FName := AName;
  FValue := AValue;
  FComment := AComment;
end;

function TMXMetafileOption.GetMetafile: TMXMetafile;
begin
  Result := FSection.Metafile;
end;

constructor TMXMetafileSection.Create(AMetafile: TMXMetafile; const AName: UTF8String;
  AComment: UTF8String = '');
begin
  FMetafile := AMetafile;
  FName := AName;
  FComment := AComment;
  FOptions := TList.Create;
end;

destructor TMXMetafileSection.Destroy;
begin
  Clear;
  FOptions.Free;
end;

function TMXMetafileSection.GetOptionCount: Integer;
begin
  Result := FOptions.Count;
end;

function TMXMetafileSection.GetOption(Idx: Integer): TMXMetafileOption;
begin
  Result := TMXMetafileOption(FOptions[Idx]);
end;

procedure TMXMetafileSection.Clear;
var i, len: Integer;
begin
  len := FOptions.Count;
  for i := 0 to len-1 do
  begin
    TMXMetafileOption(FOptions[i]).Free;
  end; // for i
  FOptions.Clear;
end;

function TMXMetafileSection.IndexOfFirstOption(const Name: UTF8String): Integer;
var i, Len: Integer;
begin
  Len := FOptions.Count;
  for i := 0 to Len-1 do
  begin
    if TMXMetafileOption(FOptions[i]).Name = Name then
    begin
      Result := i;
      Exit;
    end; // if
  end; // for i
  Result := -1;
end;

function TMXMetafileSection.GetFirstOption(const Name: UTF8String): TMXMetafileOption;
var Idx: Integer;
begin
  Idx := IndexOfFirstOption(Name);
  if Idx >= 0 then Result := TMXMetafileOption(FOptions[Idx])
  else Result := nil;
end;

function TMXMetafileSection.AddOption(const Option, Value: UTF8String): Integer;
begin
  Result := FOptions.Add(TMXMetafileOption.Create(Self, Option, Value));
end;

function TMXMetafileSection.AddOption(const Option, Value: UTF8String; const Comment: UTF8String): Integer;
begin
  Result := FOptions.Add(TMXMetafileOption.Create(Self, Option, Value, Comment));
end;

procedure TMXMetafileSection.DeleteOption(Idx: Integer);
begin
  TMXMetafileOption(FOptions[Idx]).Free;
  FOptions.Delete(Idx);
end;

procedure TMXMetafileSection.RemoveOption(AOption: TMXMetafileOption);
var Idx: Integer;
begin
  Idx := FOptions.IndexOf(AOption);
  if Idx >= 0 then
  begin
    AOption.Free;
    FOptions.Delete(Idx);
  end; // if
end;

procedure WriteComment(var AFile: TextFile; const Comment: UTF8String);
var
  Lines: TStringArray;
  i, Len: Integer;
begin
  if Trim(Comment) = '' then Exit;
  Lines := Explode(#10, Comment);
  Len := Length(Lines);
  writeln(AFile);
  for i := 0 to Len-1 do
  begin
    writeln(AFile, '# ' + Lines[i]);
  end; // for i
end;

procedure TMXMetafileSection.SaveToFile(var AFile: TextFile);
var 
  i, len: Integer;
  AOption: TMXMetafileOption;
begin
  WriteComment(AFile, Comment);
  writeln(AFile, '[' + Name + ']');
  len := OptionCount;
  for i := 0 to len-1 do
  begin
    AOption := Option[i];
    WriteComment(AFile, AOption.Comment);
    writeln(AFile, AOption.Name + ' = ' + AOption.Value);
  end; // for i
  writeln(AFile);
end;

constructor TMXMetafile.Create;
begin
  FSections := TStringList.Create;
end;

destructor TMXMetafile.Destroy;
begin
  Clear;
  FSections.Free;
end;

function TMXMetafile.GetSectionCount: Integer;
begin
  Result := FSections.Count;
end;

function TMXMetafile.GetSectionByIdx(Idx: Integer): TMXMetafileSection;
begin
  Result := TMXMetafileSection(FSections.Objects[Idx]);
end;

function TMXMetafile.GetSectionByName(const Name: UTF8String): TMXMetafileSection;
var Idx: Integer;
begin
  Idx := FSections.IndexOf(Name);
  Assert(Idx >= 0, 'section not found');
  Result := TMXMetafileSection(FSections.Objects[Idx]);
end;

function TMXMetafile.GetSectionSize(const Section: UTF8String): Integer;
begin
  Result := GetSectionByName(Section).OptionCount;
end;

function TMXMetafile.GetOption(const Section: UTF8String; OptionIdx: Integer): TMXMetafileOption;
begin
  Result := GetSectionByName(Section).Option[OptionIdx];
end;

procedure TMXMetafile.Clear;
var i, len: Integer;
begin
  len := FSections.Count;
  for i := 0 to len-1 do
  begin
    TMXMetafileSection(FSections.Objects[i]).Free;
  end; // for i
  FSections.Clear;
end;

function TMXMetafile.IndexOfSection(const Section: UTF8String): Integer;
begin
  Result := FSections.IndexOf(Section);
end;

function TMXMetafile.LookupSection(const Section: UTF8String): TMXMetafileSection;
var Idx: Integer;
begin
  Idx := FSections.IndexOf(Section);
  if Idx >= 0 then
  begin
    Result := TMXMetafileSection(FSections.Objects[Idx]);
  end
  else
  begin
    Result := TMXMetafileSection.Create(Self, Section);
    FSections.AddObject(Section, Result);
  end; // if
end;

function TMXMetafile.LookupSectionIdx(const Section: UTF8String): Integer;
begin
  Result := FSections.IndexOf(Section);
  if Result < 0 then
  begin
    Result := FSections.AddObject(Section, TMXMetafileSection.Create(Self, Section));
  end; // if
end;

procedure TMXMetafile.DeleteSection(Idx: Integer);
begin
  TMXMetafileSection(FSections.Objects[Idx]).Free;
  FSections.Delete(Idx);
end;

procedure TMXMetafile.RemoveSection(const Section: UTF8String);
var Idx: Integer;
begin
  Idx := FSections.IndexOf(Section);
  if Idx >= 0 then 
  begin
    DeleteSection(Idx);
  end; // if
end;

procedure TMXMetafile.RemoveSection(ASection: TMXMetafileSection);
begin
  Assert(ASection.Metafile = Self, 'remove strange section');
  RemoveSection(ASection.Name);
end;

procedure TMXMetafile.AddOption(const Section, Option, Value: UTF8String);
begin
  LookupSection(Section).AddOption(Option, Value);
end;

procedure TMXMetafile.AddOption(const Section, Option, Value, Comment: UTF8String);
begin
  LookupSection(Section).AddOption(Option, Value, Comment);
end;

procedure TMXMetafile.DeleteOption(const Section: UTF8String; Idx: Integer);
begin
  GetSectionByName(Section).DeleteOption(Idx);
end;

procedure TMXMetafile.RemoveOption(AOption: TMXMetafileOption);
begin
  Assert(AOption.Metafile = Self, 'remove strange option');
  AOption.Section.RemoveOption(AOption);
end;

procedure TMXMetafile.LoadFromFile(const FileName: string);
var FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmOpenRead);
  try
    LoadFromStream(FileStream);
  finally
    FileStream.Free;
  end;
end;

function BuildComment(List: TStringList): UTF8String;
var
  i, k, Count, Len: Integer;
  j: Integer;
  p: PChar;
const
  ln: UTF8String = LineEnding;
begin
  Count := List.Count;
  if Count = 0 then
  begin
    Result := '';
    Exit;
  end; // if
  Len := (Count-1) * Length(LineEnding);
  for i := 0 to Count-1 do Inc(Len, Length(List[i]));
  SetLength(Result, Len);
  p := PChar(Result);
  for i := 0 to Count-2 do
  begin
    k := Length(List[i]);
    System.Move(pchar(List[i])^, p^, k);
    for j := 1 to length(LineEnding) do
    begin
      p[k] := ln[j];
      Inc(k);
    end; // for j
    Inc(p, k);
  end; // if
  System.Move(pchar(List[Count-1])^, p^, Length(List[Count-1]));
end;

procedure TMXMetafile.LoadFromStream(Stream: TStream);
var
  Content: UTF8String;
  Lines: TStringArray;
  i, Len: Integer;
  Line: UTF8String;
  Comment: TStringList;
  Section: TMXMetafileSection;
  k: Integer;
  opt, val: UTF8String;
begin
  Clear;
  SetLength(Content, Stream.Size);
  Stream.ReadBuffer(Content[1], Stream.Size);
  Lines := Explode(#10, Content);
  Len := Length(Lines);
  Comment := TStringList.Create;
  Section := nil;
  for i := 0 to Len-1 do
  begin
    Line := trim(Lines[i]);
    if Line = '' then Continue;
    if Line[1] = '#' then
    begin
      Delete(Line, 1, 1);
      if copy(Line, 1, 1) = ' ' then Delete(Line, 1, 1);
      Comment.Add(Line);
      Continue;
    end; // if
    if Line[1] = '[' then
    begin
      assert(Line[Length(Line)] = ']', 'wrong section header');
      Section := LookupSection(copy(Line, 2, Length(Line)-2));
      Section.Comment := BuildComment(Comment);
      Comment.Clear;
      Continue;
    end; // if
    assert(Section <> nil, 'section not opened');
    k := pos('=', Line);
    assert(k > 0, 'wrong option definision');
    opt := trim(copy(Line, 1, k-1));
    val := trim(copy(Line, k+1, length(Line) - k));
    assert(opt <> '', 'wrong option name');
    Section.AddOption(opt, val, BuildComment(Comment));
    Comment.Clear;
  end;
  Comment.Free;
end;

procedure TMXMetafile.SaveToFile(const FileName: string);
var FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(FileStream);
  finally
    FileStream.Free;
  end;
end;

procedure SaveCommentToStream(Stream: TStream; const Comment: UTF8String);
var
  Lines: TStringArray;
  i, Len: Integer;
  Line: UTF8String;
begin
  Lines := Explode(#10, Comment);
  Len := Length(Lines);
  for i := 0 to Len-1 do
  begin
    Line := '# ' + Lines[i] + LineEnding;
    Stream.WriteBuffer(pchar(Line)^, Length(Line));
  end; // for i
end;

procedure TMXMetafile.SaveToStream(Stream: TStream);
var
  s_idx, s_count: Integer;
  o_idx, o_count: Integer;
  FirstSection, FirstOption: Boolean;
  Line: UTF8String;
  Section: TMXMetafileSection;
  Option: TMXMetafileOption;
begin
  s_count := SectionCount;
  FirstSection := true;
  for s_idx := 0 to s_count-1 do
  begin
    Section := GetSectionByIdx(s_idx);
    
    if not FirstSection then Stream.WriteBuffer(pchar(LineEnding)^, Length(LineEnding));
    FirstSection := false;
    if Trim(Section.Comment) <> '' then SaveCommentToStream(Stream, Section.Comment);
    
    Line := '[' + Section.Name + ']' + LineEnding;
    Stream.WriteBuffer(pchar(Line)^, Length(Line));
    FirstOption := true;
    o_count := Section.OptionCount;
    for o_idx := 0 to o_count-1 do
    begin
      Option := Section.Option[o_idx];
      if not FirstOption and (Trim(Option.Comment) <> '') then Stream.WriteBuffer(pchar(LineEnding)^, Length(LineEnding));
      FirstOption := false;
      if Trim(Option.Comment) <> '' then SaveCommentToStream(Stream, Option.Comment);
      Line := Option.Name + ' = ' + Option.Value + LineEnding;
      Stream.WriteBuffer(pchar(Line)^, Length(Line));
    end; // for o_idx
  end; // for s_idx
end;

end.