program demo_buffered;

(**
* Демонстрация TMXBufferedInput и TMXBufferedOutput
* 
* Эти два класса позволяют оптимизировать операции чтения/записи
* большого числа маленьких значений. Класс TMXBufferedInput старается
* читать данные большими блоками и все операции чтения производить
* из внутренего буфера. Класс TMXBufferedOutput накапливает все
* записываемые данные в своем внутреннем буфере и записывает их только
* когда буфер заполняется полностью или когда программист явно вызывает
* метод Flush.
* 
* Запустив эту программу вы сможете оценить скорость работы с буферизацией
* и без буферизации.
* 
* Замечание. Буферизация имеет смысл только когда записывается/читается большое
* число мелких значений. Если данные читаются/записываются большими блоками
* или их число не велико, то и выгода от использования буферизации не столь заметна
* 
* (с) Золотов Алексей, 2008 г
*     zolotov-alex@shamangrad.net
*     http://shamangrad.net/
*)

{$mode objfpc}
{$h+}
{$assertions on}

uses SysUtils, DateUtils, Classes, mxfilters;

const
  size = 2000009;

(**
* Открыть файл на запись
* 
* Если файл существует, то он обрезается, если не существует, то создается
* 
* @param FileName имя файла
* @param Buffered true - буферизовать запись
* @return файловый поток
*)
function create_file(const FileName: string; Buffered: Boolean): TStream;
var
  fs: TFileStream;
begin
  fs := TFileStream.Create(FileName, fmCreate);
  if Buffered then
  begin
    try
      // при удалении TMXBufferedOutput автоматически будет удаляться
      // нижележащий TFileStream
      Result := TMXBufferedOutput.Create(fs, true);
    except
      fs.Free;
    end;
  end
  else
  begin
    Result := fs;
  end; // if
end;

(**
* Открыть файл для чтения
* 
* @param FileName имя файла
* @param Buffered true буферизовать чтение
* @return поток
*)
function open_file(const FileName: string; Buffered: Boolean): TStream;
var
  fs: TFileStream;
begin
  fs := TFileStream.Create(FileName, fmOpenRead);
  if Buffered then
  begin
    try
      // при удалении TMXBufferedOutput автоматически будет удаляться
      // нижележащий TFileStream
      Result := TMXBufferedInput.Create(fs, true);
    except
      fs.Free;
    end;
  end
  else
  begin
    Result := fs;
  end; // if
end;

procedure test_write(Buffered: boolean);
var
  stream: TStream;
  i: Integer;
begin
  stream := create_file('test.bin', Buffered);
  try
    for i := 0 to size-1 do
    begin
      stream.WriteBuffer(i, sizeof(i));
    end; // for i
  finally
    stream.Free;
  end;
end;

procedure test_read(Buffered: Boolean);
var
  stream: TStream;
  i, x: Integer;
begin
  stream := open_file('test.bin', Buffered);
  try
    for i := 0 to size-1 do
    begin
      stream.ReadBuffer(x, sizeof(x));
      if i <> x then
      begin
        writeln('error: ', x, ' <> ', i);
        break;
      end;
    end; // for i
  finally
    stream.Free;
  end;
end;

var
  tm: TDateTime;

procedure start_time;
begin
  tm := now;
end;

function end_time: Int64;
begin
  Result := MilliSecondsBetween(now, tm);
end;

begin
  writeln('stage 1: unbuffered write');
  start_time;
  test_write(false);
  writeln('time: ', end_time, ' ms');
  writeln;
  
  writeln('stage 2: unbuffered read');
  start_time;
  test_read(false);
  writeln('time: ', end_time, ' ms');
  writeln;
  
  writeln('stage 3: buffered write');
  start_time;
  test_write(true);
  writeln('time: ', end_time, ' ms');
  writeln;
  
  writeln('stage 4: buffered read');
  start_time;
  test_read(true);
  writeln('time: ', end_time, ' ms');
  writeln;
  
  writeln('done');
end.