program demo_mxfilters;

(**
* Демонстрация потоковых фильтров
* 
* Потоковые фильтры позволяют произвольным образом преобразовывать
* читаемые/записываемые данные: шифровать, сжимать, буферизовать и т.п.
* Потоковые фильтры можно выстраивать в цепочку комбинируя несколько
* разных фильтров добиваться требуемых результатов. Например, если
* есть фильтры буферизации, сжатия и шифрования то их можно выстроить
* в цепочку:
*   буферизация -> сжатие -> шифрование -> запись в файл
* 
*   FileStream := TFileStream.Create(FileName, fmCreate) - создаем поток
*      файла для записи
*   CryptStream := TCryptStream.Create(fs) - создаем потоковый фильтр
*      ширования, результат шифрования будет записываться в поток FileStream
*   CompressStream := TCompressStream.Create(CryptStream) - создаем потоковый
*      фильтр сжатия, результат сжатия будет записываться в поток CryptStream
*   BufferedStream := TMXBufferedOutput.Create(CompressStream) - создаем потоковый
*      фильтр буферизации, результат будет записываться в поток CompressStream
*
*   Таким образом все данные записываемые в поток BufferedStream будет сначала
*   накапливаться во внутреннем буфере, оптимизируя операции записи большого числа
*   маленьких блоков информации, сжимать затем сжимать их, шифровать и записывать
*   в файл. Последний фильтр вы можете использовать везде где требуется объект
*   класса TStream
*
*
* Примечание. Все фильтры используют последовательную запись и последовательное
* чтение информации, в них нельзя использовать методы Seek и свойство Position
* (для многих потоков они просто не применимы)
* 
* Примечание. Юнит mxfilters предоставляет только базовые классы для создания
* собственных фильтров и несколько фильтров общего назначения: TMXBufferedOutput
* и TMXBufferedInput.
* 
* (с) Золотов Алексей, 2008 г
*     zolotov-alex@shamangrad.net
*     http://shamangrad.net/
*)

{$mode objfpc}
{$h+}
{$assertions on}

uses SysUtils, Classes, mxfilters;

type
  (**
  * Простейший пример шифрующего/дешифрующего фильтра
  *)
  TCryptStream = class (TMXFilter)
    public
      Key: Byte;
      
      (**
      * Прочитать блок данных из защифрованного потока
      *
      * @param buf буфер для читаемых данных
      * @param len длина читаемого блока
      * @return число реально прочитанных байт
      *)
      function Read(var buf; len: Longint): Longint; override;
      
      (**
      * Записать блок данных в защифрованный поток
      *
      * @param buf буфер для записываемых данных
      * @param len длина записываемого блока
      * @return число реально записанных байт
      *)
      function Write(const buf; len: Longint): Longint; override;
  end;

function TCryptStream.Read(var buf; len: Longint): Longint;
var
  p: pbyte;
  i: Longint;
begin
  Result := FStream.Read(buf, len);
  p := pbyte(@buf);
  for i := 0 to Result-1 do
  begin
    p[i] := p[i] - Key;
  end; // for i
end;

function TCryptStream.Write(const buf; len: Longint): Longint;
var
  p, cbuf: pbyte;
  i: Longint;
begin
  GetMem(cbuf, len);
  try
    p := pbyte(@buf);
    for i := 0 to len-1 do
    begin
      cbuf[i] := p[i] + Key;
    end; // for i
    Result := FStream.Write(cbuf^, len);
  finally
    FreeMem(cbuf);
  end;
end;

(**
* Пример записи в поток зашифрованной строки
*
* @param Stream поток
* @param Key ключ шифрования
* @param Message записываемая строка
*)
procedure CryptToStream(Stream: TStream; Key: Byte; Message: string);
var
  cs: TCryptStream;
  len: Longint;
begin
  cs := TCryptStream.Create(Stream, false);
  try
    cs.Key := Key;
    len := length(Message);
    cs.writebuffer(len, sizeof(len));
    cs.writebuffer(Message[1], len);
  finally
    cs.Free;
  end;
end;

(**
* Пример чтения из потока защифрованной строки
*
* @param Stream поток
* @param Key ключ шифрования
* @return прочитанная строка
*)
function DecryptFromStream(Stream: TStream; Key: Byte): string;
var
  cs: TCryptStream;
  len: Longint;
begin
  cs := TCryptStream.Create(Stream, false);
  try
    cs.Key := Key;
    cs.ReadBuffer(len, sizeof(len));
    SetLength(Result, len);
    if len > 0 then cs.ReadBuffer(Result[1], len);
  finally
    cs.Free;
  end;
end;

(**
* Пример записи зашифрованной строки в файл
* 
* @param FileName имя файла
* @param Key ключ шифрования
* @param Message записываемая строка
*)
procedure CryptToFile(const FileName: string; Key: Byte; const Message: string);
var
  cs: TCryptStream;
  len: Longint;
begin
  cs := TCryptStream.Create(TFileStream.Create(FileName, fmCreate), true);
  try
    cs.Key := Key;
    len := length(Message);
    cs.writebuffer(len, sizeof(len));
    cs.writebuffer(Message[1], len);
  finally
    cs.Free;
  end;
end;

(**
* Пример открытия зашифрованного файла
* 
* @param FileName имя файла
* @param Key ключ шифрования
* @return расшифрованный поток
*)
function OpenCryptedFile(const FileName: string; Key: Byte): TStream;
var
  fs: TStream;
  cs: TCryptStream;
begin
  fs := TFileStream.Create(FileName, fmOpenRead);
  try
    // при удалении TCryptStream будет удален нижележащий TFileStream
    cs := TCryptStream.Create(fs, true);
  except
    fs.Free;
  end;
  
  cs.Key := Key;
  Result := cs;
end;

(**
* Пример чтения зашифрованной строки из файла
* 
* @param FileName имя файла
* @param Key ключ шифрования
* @return прочитанная строка
*)
function DecryptFromFile(const FileName: string; Key: Byte): string;
var
  cs: TStream;
  len: Longint;
begin
  cs := OpenCryptedFile(FileName, Key);
  try
    cs.ReadBuffer(len, sizeof(len));
    SetLength(Result, len);
    if len > 0 then cs.ReadBuffer(Result[1], len);
  finally
    cs.Free;
  end;
end;

var
  message: string;
begin
  write('input message: ');
  readln(message);
  CryptToFile('test.bin', 3, Message);
  writeln('message crypted');
  writeln;
  message := DecryptFromFile('test.bin', 3);
  writeln('decrypted message: ', message);
end.