unit mxfilters;

(******************************************************************************
  mxfilters.pas
    Some base classes for cunstruction custom stream filters
  
  Copyright notice:

 (c) 2008 Zolotov Alex

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Zolotov Alex, 2008
    zolotov-alex@shamangrad.net
    http://shamangrad.net/

 ******************************************************************************)

{$mode objfpc}
{$h+}
{$assertions on}

interface
uses SysUtils, Classes;

type
  (**
  * Базовый абстрактный класс фильтра-потока
  * 
  * Позволяет преобразовывать как входные так и выходные данные
  *)
  TMXFilter = class (TStream)
    protected
      FStream: TStream;
      FFreeOnDestory: Boolean;
      
      (**
      * Инициализация потока
      *
      * Переопределите этот метод, если нужна какая-то инициальзация потока
      *)
      procedure InitStream; virtual;
      
      (**
      * Финализация потока
      *
      * Переопределите этот метод, если нужна какая-то финализация потока
      *)
      procedure FinalizeStream; virtual;
    public
      (**
      * Конструктор
      * @param AStream нижележащий поток
      * @param FreeOnDestroy - true объект нижележащего потока будет удален
      *   при удалении потока
      *
      * @note Переопределять не рекомендуется. Если необходима инициализация,
      *   переопределите метод InitStream()
      *)
      constructor Create(AStream: TStream; FreeOnDestroy: Boolean = false); virtual;
      
      (**
      * Деструктор
      *
      * @note Переопределять не рекомендуется. Если необходима финализация,
      *   переопределите метод FinalizeStream()
      *)
      destructor Destroy; override;
      
      (**
      * Метод-заглушка
      *
      * Всегда вызывает исключение, т.к. фильтры используют только потоковую
      * запись/чтения и не позволяют перемещаться по потоку
      *
      * @note переопределять не рекомендуется
      *)
      function Seek(Offset: Longint; Origin: Word): Longint; override; overload;
      
      (**
      * Метод-заглушка
      *
      * Всегда вызывает исключение, т.к. фильтры используют только потоковую
      * запись/чтения и не позволяют перемещаться по потоку
      *
      * @note переопределять не рекомендуется
      *)
      function Seek(const Offset: Int64; Origin: TSeekOrigin): Int64; override; overload;
      
      (**
      * Записать данные накопленные выходном буфере в нижележащий поток
      *)
      procedure Flush; virtual;
  end;
  
  (**
  * Базовый абстрактный класс входного фильтра-потока
  *
  * Позволяет только читать данные из потока
  *)
  TMXInputFilter = class (TMXFilter)
    public
      (**
      * Метод-заглушка
      *
      * Всегда вызывает исключение, т.к. класс TMXInputFilter используется
      * только для чтения
      *)
      function Write(const buf; len: Longint): Longint; override;
  end;
  
  (**
  * Базовый абстрактный класс выходного фильтра-потока
  *
  * Позволяет только писать данные в поток
  *)
  TMXOutputFilter = class (TMXFilter)
    public
      (**
      * Метод-заглушка
      *
      * Всегда вызывает исключение, т.к. класс TMXOutputFilter используется
      * только для записи
      *)
      function Read(var buf; len: Longint): Longint; override;
  end;
  
  (**
  * Входной буферезующий фильтр
  *
  * Данный класс пытается читать данные из нижележащего потока
  * большими блоками, чтобы оптимизировать чтение маленькими
  * блоками на медленных нижележащий потоков (если нижележащий
  * поток эффективнее работает с большими блоками)
  *)
  TMXBufferedInput = class (TMXInputFilter)
    private
      FBuffer: pbyte; // буфер
      FBufferSize: Integer; // размер буфера (вместительность)
      FUnreadOffset: Integer; // смещение в буфере к не прочитаному блоку
      FUnreadSize: Integer; // размер не прочитанного блока
    protected
      (**
      * Инициализация буферизующего потока
      *)
      procedure InitStream; override;
      
      (**
      * Финализация буферизующего потока
      *)
      procedure FinalizeStream; override;
      
      (**
      * Изменить размер буфера
      * 
      * @note уменьшить размер буфера можно только когда буфер пуст
      *)
      procedure SetBufferSize(NewBufferSize: Integer);
      
      (**
      * Прочитать блок данных из внутреннего буфера
      * 
      * @param buf указатель на выходной буфер
      * @param len сколько байт нужно прочитать
      * @return число реально прочитано байт, если меньше чем len,
      *   то внутренний буфер пуст
      *)
      function ReadFromBuffer(buf: pbyte; len: Longint): Longint;
    public
      (**
      * Чтение блока данных
      *
      * @note переопределять не рекомендуется
      *)
      function Read(var buf; len: Longint): Longint; override;
      
      (**
      * Размер буфера
      *
      * По умолчанию размер буфера равен 4096 байт
      *
      * @note уменьшить размер буфера можно только когда буфер пуст
      *)
      property BufferSize: Integer read FBufferSize write SetBufferSize;
  end;
  
  (**
  * Выходной буферезующий фильтр
  *
  * Данный класс накапливает записываемые данные во внутренем буфере,
  * когда буфер полностью заполняется, то данные из буфера записываются
  * в нижележащий поток. Чтобы принудительно записать данные накопленые
  * в буфере используйте метод Flush
  *)
  TMXBufferedOutput = class (TMXOutputFilter)
    private
      FBuffer: pbyte; // буфер
      FBufferSize: Longint; // размер буфера
      FDataSize: Longint; // размер данных накопленных в буфере
    protected
      (**
      * Инициализация буферизующего потока
      *)
      procedure InitStream; override;
      
      (**
      * Финализация буферизующего потока
      *)
      procedure FinalizeStream; override;
      
      (**
      * Изменить размер буфера
      * 
      * @param NewBufferSize новый размер буфера
      *)
      procedure SetBufferSize(NewBufferSize: Integer);
      
      (**
      * Записать блок данных во внутренний буфер
      * 
      * @param buf указатель на входной буфер
      * @param len сколько байт нужно записать
      * @return число реально записанных байт, если меньше чем len,
      *   то внутрений буфер полон
      *)
      function WriteToBuffer(buf: pbyte; len: Longint): Longint;
      
      (**
      * Записать данные внутреннего буфера в нижележащий поток
      * 
      * @return true если удалось записать хотя бы один байт
      *)
      function SaveBuffer: Boolean;
    public
      (**
      * Запись блока данных
      *
      * @note переопределять не рекомендуется
      *)
      function Write(const buf; len: Longint): Longint; override;
      
      (**
      * Записать данные накопленные выходном буфере в нижележащий поток
      *)
      procedure Flush; override;
      
      (**
      * Размер буфера
      *
      * По умолчанию размер буфера равен 4096 байт
      *)
      property BufferSize: Integer read FBufferSize write SetBufferSize;
  end;


implementation

constructor TMXFilter.Create(AStream: TStream; FreeOnDestroy: Boolean);
begin
  inherited Create;
  FStream := AStream;
  FFreeOnDestory := FreeOnDestroy;
  InitStream;
end;

destructor TMXFilter.Destroy;
begin
  Flush;
  FinalizeStream;
  if FFreeOnDestory then
  begin
    FStream.Free;
  end; // if
  inherited Destroy;
end;

procedure TMXFilter.InitStream;
begin
end;

procedure TMXFilter.FinalizeStream;
begin
end;

function TMXFilter.Seek(Offset: Longint; Origin: Word): Longint;
begin
  raise Exception.Create('TMXFilter.Seek() not supported');
  Result := 0;
end;

function TMXFilter.Seek(const Offset: Int64; Origin: TSeekOrigin): Int64;
begin
  raise Exception.Create('TMXFilter.Seek() not supported');
  Result := 0;
end;

procedure TMXFilter.Flush;
begin
  // по умолчанию буфера нет, записывать нечего
end;

function TMXInputFilter.Write(const buf; len: Longint): Longint;
begin
  raise Exception.Create('TMXInputFilter.Write() not supported');
  Result := 0;
end;

function TMXOutputFilter.Read(var buf; len: Longint): Longint;
begin
  raise Exception.Create('TMXOutputFilter.Read() not supported');
  Result := 0;
end;

procedure TMXBufferedInput.InitStream;
begin
  FBufferSize := 4096;
  FUnreadSize := 0;
  GetMem(FBuffer, FBufferSize);
end;

procedure TMXBufferedInput.FinalizeStream;
begin
  FreeMem(FBuffer);
end;

procedure TMXBufferedInput.SetBufferSize(NewBufferSize: Longint);
begin
  if (NewBufferSize > FBufferSize) or (FUnreadSize > 0) then
  begin
    FBuffer := ReallocMem(FBuffer, NewBufferSize);
    FBufferSize := NewBufferSize;
  end; // if
end;

function TMXBufferedInput.ReadFromBuffer(buf: pbyte; len: Longint): Longint;
begin
  // Если в буфере достаточно данных, то читаем из буфера и выходим
  if FUnreadSize >= len then
  begin
    Move(FBuffer[FUnreadOffset], buf^, len);
    FUnreadSize := FUnreadSize - len;
    FUnreadOffset := FUnreadOffset + len;
    Result := len;
    Exit;
  end; // if
  
  // В буфере недостаточно данных, читаем то, что есть в буфере
  if FUnreadSize > 0 then
  begin
    Move(FBuffer[FUnreadOffset], buf^, FUnreadSize);
    Result := FUnreadSize;
    FUnreadSize := 0;
  end
  else
  begin
    Result := 0;
  end; // if
end;

function TMXBufferedInput.Read(var buf; len: Longint): Longint;
var
  count: Longint;
  pbuf: pbyte;
begin
  assert(len >= 0);
  
  pbuf := pbyte(@buf); // указатель на выходной буфер
  Result := 0; // число байт, которые удалось прочитать
  // len - число байт, которые осталось прочитать
  
  while len > 0 do
  begin
    count := ReadFromBuffer(pbuf, len);
    len := len - count;
    pbuf := pbuf + count;
    Result := Result + count;
    
    if len > 0 then
    begin
      // буфер пуст, если требуется больше, чем может уместиться в буфере,
      // то читаем без буфера и выходим
      if len > FBufferSize then
      begin
        Result := Result + FStream.Read(pbuf^, len);
        Exit;
      end; // if
      
      // заполним пустой буфер
      FUnreadOffset := 0;
      FUnreadSize := FStream.Read(FBuffer^, FBufferSize);
    end; // if
  end; // while
end;

procedure TMXBufferedOutput.InitStream;
begin
  FBufferSize := 4096;
  FDataSize := 0;
  GetMem(FBuffer, FBufferSize);
end;

procedure TMXBufferedOutput.FinalizeStream;
begin
  FreeMem(FBuffer);
end;

procedure TMXBufferedOutput.SetBufferSize(NewBufferSize: Longint);
begin
  if (NewBufferSize > FBufferSize) or (FDataSize = 0) then
  begin
    FBuffer := ReallocMem(FBuffer, NewBufferSize);
    FBufferSize := NewBufferSize;
  end; // if
end;

function TMXBufferedOutput.WriteToBuffer(buf: pbyte; len: Longint): Longint;
var FreeSize: Longint;
begin
  FreeSize := FBufferSize - FDataSize;
  
  // Если в буфере достаточно достаточно места, то записываем и выходим
  if FreeSize >= len then
  begin
    Move(buf^, FBuffer[FDataSize], len);
    FDataSize := FDataSize + len;
    Result := len;
    Exit;
  end; // if
  
  // В буфере недостаточно места, запишем то, что сможем
  if FreeSize > 0 then
  begin
    Move(buf^, FBuffer[FDataSize], FreeSize);
    FDataSize := FBufferSize;
    Result := FreeSize;
  end
  else
  begin
    Result := 0;
  end; // if
end;

function TMXBufferedOutput.SaveBuffer: Boolean;
var count: Longint;
begin
  // Если нечего записывать, то будем считать, что удалось записать
  if FDataSize = 0 then
  begin
    Result := true;
    Exit;
  end; // if
  
  count := FStream.Write(FBuffer^, FDataSize);
  
  // Если ничего не удалось записать, то выходим
  Result := count <> 0;
  if not Result then Exit;
  
  // Если удалось записать только часть данных, то нужно подвинуть буфер
  if count < FDataSize then
  begin
    Move(FBuffer[count], FBuffer^, FDataSize - count);
  end; // if
  FDataSize := FDataSize - count;
end;

function TMXBufferedOutput.Write(const buf; len: Longint): Longint;
var
  count: Longint;
  pbuf: pbyte;
begin
  assert(len >= 0);
  
  pbuf := pbyte(@buf); // указатель на входной буфер
  Result := 0; // число байт, которые удалось записать
  // len - число байт, которые осталось записать
  
  while len > 0 do
  begin
    // Если буфер пуст и нужно записать больше, чем может уместиться
    // в буфере, то попробуем сначала записать без буфера
    if (FDataSize = 0) and (len > FBufferSize) then
    begin
      count := FStream.Write(pbuf^, len);
      Result := Result + count;
      if count = len then Exit;
      len := len - count;
      pbuf := pbuf + count;
    end; // if
    
    // пробуем записать в буфер
    count := WriteToBuffer(pbuf, len);
    Result := Result + count;
    
    if count < len then
    begin
      // буфер полон, запишем буферизованые данные в нижележащий поток
      // eсли ничего записать не получается, то выходим
      if not SaveBuffer then Exit;
    end; // if
    
    len := len - count;
    pbuf := pbuf + count;
  end; // while
end;

procedure TMXBufferedOutput.Flush;
begin
  if not SaveBuffer then
  begin
    raise Exception.Create('TMXBufferedOutput.Flush fault');
  end;
end;

end.
