
(******************************************************************************
  svn_auth.inc

  Export of libsvn_auth

  Zolotov Alex, 2007
    mail: zolotov-alex@mail.ru
    url: http://zolotov.h14.ru

 *****************************************************************************)

const
(** Universal run-time parameters, made available to all providers.

    If you are writing a new provider, then to be a 'good citizen',
    you should notice these global parameters!  Note that these
    run-time params should be treated as read-only by providers; the
    application is responsible for placing them into the auth_baton
    hash. *)

(** The auth-hash prefix indicating that the parameter is global. *)
  SVN_AUTH_PARAM_PREFIX = 'svn:auth:';

(**
 * @name Default credentials defines
 * Any 'default' credentials that came in through the application itself,
 * (e.g. --username and --password options). Property values are
 * const char *.
 * @{ *)
 SVN_AUTH_PARAM_DEFAULT_USERNAME: pchar = SVN_AUTH_PARAM_PREFIX + 'username';
 SVN_AUTH_PARAM_DEFAULT_PASSWORD: pchar = SVN_AUTH_PARAM_PREFIX + 'password';
(** @} *)

(** @brief The application doesn't want any providers to prompt
 * users. Property value is irrelevant; only property's existence
 * matters. *)
 SVN_AUTH_PARAM_NON_INTERACTIVE: pchar = SVN_AUTH_PARAM_PREFIX + 'non-interactive';

(** @brief The application doesn't want any providers to save passwords
 * to disk. Property value is irrelevant; only property's existence
 * matters. *)
 SVN_AUTH_PARAM_DONT_STORE_PASSWORDS: pchar = SVN_AUTH_PARAM_PREFIX + 'dont-store-passwords';

(** @brief The application doesn't want any providers to save credentials
 * to disk. Property value is irrelevant; only property's existence
 * matters. *)
 SVN_AUTH_PARAM_NO_AUTH_CACHE: pchar = SVN_AUTH_PARAM_PREFIX + 'no-auth-cache';

(** @brief The following property is for SSL server cert providers. This
 * provides a pointer to an @c apr_uint32_t containing the failures
 * detected by the certificate validator. *)
 SVN_AUTH_PARAM_SSL_SERVER_FAILURES: pchar = SVN_AUTH_PARAM_PREFIX + 'ssl:failures';

(** @brief The following property is for SSL server cert providers. This
 * provides the cert info (svn_auth_ssl_server_cert_info_t). *)
 SVN_AUTH_PARAM_SSL_SERVER_CERT_INFO: pchar = SVN_AUTH_PARAM_PREFIX + 'ssl:cert-info';

(** Some providers need access to the @c svn_config_t configuration. *)
 SVN_AUTH_PARAM_CONFIG: pchar = SVN_AUTH_PARAM_PREFIX + 'config';

(** The current server group. *)
 SVN_AUTH_PARAM_SERVER_GROUP: pchar = SVN_AUTH_PARAM_PREFIX + 'server-group';

(** @brief A configuration directory that overrides the default
 * ~/.subversion. *)
 SVN_AUTH_PARAM_CONFIG_DIR: pchar = SVN_AUTH_PARAM_PREFIX + 'config-dir';
  
  {* Overview of the svn authentication system.
   *
   * We define an authentication 'provider' as a module that is able to
   * return a specific set of credentials. (e.g. username/password,
   * certificate, etc.)  Each provider implements a vtable that
   *
   * - can fetch initial credentials
   * - can retry the fetch (or try to fetch something different)
   * - can store the credentials for future use
   *
   * For any given type of credentials, there can exist any number of
   * separate providers -- each provider has a different method of
   * fetching. (i.e. from a disk store, by prompting the user, etc.)
   *
   * The application begins by creating an auth baton object, and
   * 'registers' some number of providers with the auth baton, in a
   * specific order.  (For example, it may first register a
   * username/password provider that looks in disk store, then register
   * a username/password provider that prompts the user.)
   *
   * Later on, when any svn library is challenged, it asks the auth
   * baton for the specific credentials.  If the initial credentials
   * fail to authenticate, the caller keeps requesting new credentials.
   * Under the hood, libsvn_auth effectively 'walks' over each provider
   * (in order of registry), one at a time, until all the providers have
   * exhausted all their retry options.
   *
   * This system allows an application to flexibly define authentication
   * behaviors (by changing registration order), and very easily write
   * new authentication providers.
   *
   * An auth_baton also contains an internal hashtable of run-time
   * parameters; any provider or library layer can set these run-time
   * parameters at any time, so that the provider has access to the
   * data.  (For example, certain run-time data may not be available
   * until an authentication challenge is made.)  Each credential type
   * must document the run-time parameters that are made available to
   * its providers.
   *
   * @defgroup auth_fns authentication functions
   * @
    }

  {*
   * Return a deep copy of @a info, allocated in @a pool.
   *
   * @since New in 1.3.
    }
  function svn_auth_ssl_server_cert_info_dup(info:Psvn_auth_ssl_server_cert_info_t; pool:apr_pool_p):Psvn_auth_ssl_server_cert_info_t;cdecl;external libsvn_auth name 'svn_auth_ssl_server_cert_info_dup';

  {* Initialize an authentication system.
   *
   * Return an authentication object in @a *auth_baton (allocated in @a
   * pool) that represents a particular instance of the svn
   * authentication system.  @a providers is an array of @c
   * svn_auth_provider_object_t pointers, already allocated in @a pool
   * and intentionally ordered.  These pointers will be stored within @a
   * *auth_baton, grouped by credential type, and searched in this exact
   * order.
    }

  procedure svn_auth_open(out auth_baton:Psvn_auth_baton_t; providers:apr_array_header_p; pool:apr_pool_p);cdecl;external libsvn_auth name 'svn_auth_open';

  {* Set an authentication run-time parameter.
   *
   * Store @a name / @a value pair as a run-time parameter in @a
   * auth_baton, making the data accessible to all providers.  @a name
   * and @a value will NOT be duplicated into the auth_baton's pool.
   * To delete a run-time parameter, pass NULL for @a value.
    }
  procedure svn_auth_set_parameter(auth_baton:Psvn_auth_baton_t; name:Pchar; value:pointer);cdecl;external libsvn_auth name 'svn_auth_set_parameter';

  {* Get an authentication run-time parameter.
   *
   * Return a value for run-time parameter @a name from @a auth_baton.
   * Return NULL if the parameter doesn't exist.
    }
  function svn_auth_get_parameter(auth_baton:Psvn_auth_baton_t; name:Pchar):pointer;cdecl;external libsvn_auth name 'svn_auth_get_parameter';

    {* Get an initial set of credentials.
     *
     * Ask @a auth_baton to set @a *credentials to a set of credentials
     * defined by @a cred_kind and valid within @a realmstring, or NULL if
     * no credentials are available.  Otherwise, return an iteration state
     * in @a *state, so that the caller can call
     * svn_auth_next_credentials(), in case the first set of credentials
     * fails to authenticate.
     *
     * Use @a pool to allocate @a *state, and for temporary allocation.
     * Note that @a *credentials will be allocated in @a auth_baton's pool.
      }

  function svn_auth_first_credentials(out credentials:pointer; var state:Psvn_auth_iterstate_t; cred_kind:Pchar; realmstring:Pchar; auth_baton:Psvn_auth_baton_t;
    pool:apr_pool_p):Psvn_error_t;cdecl;external libsvn_auth name 'svn_auth_first_credentials';

    {* Get another set of credentials, assuming previous ones failed to
     * authenticate.
     *
     * Use @a state to fetch a different set of @a *credentials, as a
     * follow-up to svn_auth_first_credentials() or
     * svn_auth_next_credentials().  If no more credentials are available,
     * set @a *credentials to NULL.
     *
     * Note that @a *credentials will be allocated in @c auth_baton's pool.
      }
  function svn_auth_next_credentials(out credentials:pointer; state:Psvn_auth_iterstate_t; pool:apr_pool_p):Psvn_error_t;cdecl;external libsvn_auth name 'svn_auth_next_credentials';

    {* Save a set of credentials.
     *
     * Ask @a state to store the most recently returned credentials,
     * presumably because they successfully authenticated.  Use @a pool
     * for temporary allocation.  If no credentials were ever returned, do
     * nothing.
      }
  function svn_auth_save_credentials(state:Psvn_auth_iterstate_t; pool:apr_pool_p):Psvn_error_t;cdecl;external libsvn_auth name 'svn_auth_save_credentials';

    {* @  }
    {* Create and return @a *provider, an authentication provider of type
     * svn_auth_cred_simple_t that gets information by prompting the user
     * with @a prompt_func and @a prompt_baton.  Allocate @a *provider in
     * @a pool.
     *
     * If both @c SVN_AUTH_PARAM_DEFAULT_USERNAME and
     * @c SVN_AUTH_PARAM_DEFAULT_PASSWORD are defined as runtime
     * parameters in the @c auth_baton, then @a *provider will return the
     * default arguments when svn_auth_first_credentials() is called.  If
     * svn_auth_first_credentials() fails, then @a *provider will
     * re-prompt @a retry_limit times (via svn_auth_next_credentials()).
     *
     * @since New in 1.4.
      }
  procedure svn_auth_get_simple_prompt_provider(out provider:svn_auth_provider_object_p;
      prompt_func:svn_auth_simple_prompt_func_t; prompt_baton:pointer;
      retry_limit:longint; pool:apr_pool_p); cdecl;
      external libsvn_auth name 'svn_auth_get_simple_prompt_provider';

    {* Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_username_t that gets information by prompting the
     * user with @a prompt_func and @a prompt_baton.  Allocate @a *provider
     * in @a pool.
     *
     * If @c SVN_AUTH_PARAM_DEFAULT_USERNAME is defined as a runtime
     * parameter in the @c auth_baton, then @a *provider will return the
     * default argument when svn_auth_first_credentials() is called.  If
     * svn_auth_first_credentials() fails, then @a *provider will
     * re-prompt @a retry_limit times (via svn_auth_next_credentials()).
     *
     * @since New in 1.4.
      }
  procedure svn_auth_get_username_prompt_provider(out provider:svn_auth_provider_object_p;
      prompt_func:svn_auth_username_prompt_func_t;
      prompt_baton:pointer; retry_limit:longint; pool:apr_pool_p);cdecl;
      external libsvn_auth name 'svn_auth_get_username_prompt_provider';

    {* Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_simple_t that gets/sets information from the user's
     * ~/.subversion configuration directory.  Allocate @a *provider in
     * @a pool.
     *
     * If a default username or password is available, @a *provider will
     * honor them as well, and return them when
     * svn_auth_first_credentials() is called.  (see @c
     * SVN_AUTH_PARAM_DEFAULT_USERNAME and @c
     * SVN_AUTH_PARAM_DEFAULT_PASSWORD).
     *
     * @since New in 1.4.
      }
  procedure svn_auth_get_simple_provider(out provider:svn_auth_provider_object_p; pool:apr_pool_p);cdecl;external libsvn_auth name 'svn_auth_get_simple_provider';

{$IFDEF WIN32}
    {*
     * Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_simple_t that gets/sets information from the user's
     * ~/.subversion configuration directory.  Allocate @a *provider in
     * @a pool.
     *
     * This is like svn_client_get_simple_provider(), except that, when
     * running on Window 2000 or newer (or any other Windows version that
     * includes the CryptoAPI), the provider encrypts the password before
     * storing it to disk. On earlier versions of Windows, the provider
     * does nothing.
     *
     * @since New in 1.4.
     * @note This function is only available on Windows.
     *
     * @note An administrative password reset may invalidate the account's
     * secret key. This function will detect that situation and behave as
     * if the password were not cached at all.
      }

  procedure svn_auth_get_windows_simple_provider(out provider:svn_auth_provider_object_p; pool:apr_pool_p);cdecl;external libsvn_auth name 'svn_auth_get_windows_simple_provider';
{$ENDIF}

{$IFDEF DARWIN}
    {*
     * Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_simple_t that gets/sets information from the user's
     * ~/.subversion configuration directory.  Allocate @a *provider in
     * @a pool.
     *
     * This is like svn_client_get_simple_provider(), except that the
     * password is stored in the Mac OS KeyChain.
     *
     * @since New in 1.4
     * @note This function is only available on Mac OS 10.2 and higher.
      }

  procedure svn_auth_get_keychain_simple_provider(out provider:svn_auth_provider_object_p; pool:apr_pool_p);cdecl;external libsvn_auth name 'svn_auth_get_keychain_simple_provider';
{$ENDIF}

    {* Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_username_t that gets/sets information from a user's
     * ~/.subversion configuration directory.  Allocate @a *provider in
     * @a pool.
     *
     * If a default username is available, @a *provider will honor it,
     * and return it when svn_auth_first_credentials() is called.  (See
     * @c SVN_AUTH_PARAM_DEFAULT_USERNAME.)
     *
     * @since New in 1.4.
      }

  procedure svn_auth_get_username_provider(out provider:svn_auth_provider_object_p; pool:apr_pool_p);cdecl;external libsvn_auth name 'svn_auth_get_username_provider';

    {* Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_ssl_server_trust_t, allocated in @a pool.
     *
     * @a *provider retrieves its credentials from the configuration
     * mechanism.  The returned credential is used to override SSL
     * security on an error.
     *
     * @since New in 1.4.
      }
  
  procedure svn_auth_get_ssl_server_trust_file_provider(out provider:svn_auth_provider_object_p; pool:apr_pool_p);cdecl;external libsvn_auth name 'svn_auth_get_ssl_server_trust_file_provider';

    {* Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_ssl_client_cert_t, allocated in @a pool.
     *
     * @a *provider retrieves its credentials from the configuration
     * mechanism.  The returned credential is used to load the appropriate
     * client certificate for authentication when requested by a server.
     *
     * @since New in 1.4.
      }
  procedure svn_auth_get_ssl_client_cert_file_provider(out provider:svn_auth_provider_object_p; pool:apr_pool_p);cdecl;external libsvn_auth name 'svn_auth_get_ssl_client_cert_file_provider';

    {* Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_ssl_client_cert_pw_t, allocated in @a pool.
     *
     * @a *provider retrieves its credentials from the configuration
     * mechanism.  The returned credential is used when a loaded client
     * certificate is protected by a passphrase.
     *
     * @since New in 1.4.
      }
  procedure svn_auth_get_ssl_client_cert_pw_file_provider(out provider:svn_auth_provider_object_p; pool:apr_pool_p);cdecl;external libsvn_auth name 'svn_auth_get_ssl_client_cert_pw_file_provider';

    {* Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_ssl_server_trust_t, allocated in @a pool.  
     *
     * @a *provider retrieves its credentials by using the @a prompt_func
     * and @a prompt_baton.  The returned credential is used to override
     * SSL security on an error.
     *
     * @since New in 1.4.
      }
  procedure svn_auth_get_ssl_server_trust_prompt_provider(out provider:svn_auth_provider_object_p;
      prompt_func:svn_auth_ssl_server_trust_prompt_func_t; prompt_baton:pointer; pool:apr_pool_p);cdecl;
      external libsvn_auth name 'svn_auth_get_ssl_server_trust_prompt_provider';

    {* Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_ssl_client_cert_t, allocated in @a pool.
     *
     * @a *provider retrieves its credentials by using the @a prompt_func
     * and @a prompt_baton.  The returned credential is used to load the
     * appropriate client certificate for authentication when requested by
     * a server.  The prompt will be retried @a retry_limit times.
     *
     * @since New in 1.4.
      }
  procedure svn_auth_get_ssl_client_cert_prompt_provider(
      out provider:svn_auth_provider_object_p;
      prompt_func:svn_auth_ssl_client_cert_prompt_func_t;
      prompt_baton:pointer; retry_limit:longint; pool:apr_pool_p);cdecl;
      external libsvn_auth name 'svn_auth_get_ssl_client_cert_prompt_provider';

    {* Create and return @a *provider, an authentication provider of type @c
     * svn_auth_cred_ssl_client_cert_pw_t, allocated in @a pool.
     *
     * @a *provider retrieves its credentials by using the @a prompt_func
     * and @a prompt_baton.  The returned credential is used when a loaded
     * client certificate is protected by a passphrase.  The prompt will
     * be retried @a retry_limit times.
     *
     * @since New in 1.4.
      }
  procedure svn_auth_get_ssl_client_cert_pw_prompt_provider(
      out provider:svn_auth_provider_object_p;
      prompt_func:svn_auth_ssl_client_cert_pw_prompt_func_t; prompt_baton:pointer;
      retry_limit:longint; pool:apr_pool_p);cdecl;
      external libsvn_auth name 'svn_auth_get_ssl_client_cert_pw_prompt_provider';
