
(******************************************************************************
  svn_client.inc

  @file svn_types.h
  @brief Subversion's data types

  Export of libsvn_client

  Zolotov Alex, 2007
    mail: zolotov-alex@mail.ru
    url: http://zolotov.h14.ru

 *****************************************************************************)

  {* Subversion error object.
   *
   * Defined here, rather than in svn_error.h, to avoid a recursive @#include 
   * situation.
    }
  psvn_error_t = ^svn_error_t;
  svn_error_p = ^svn_error_t;
  svn_error_t = record
    apr_err: apr_status_t;
    message: pchar;
    child: svn_error_p;
    pool: apr_pool_p;
    _file: pchar;
    line: longint;
  end;


    {* @  }
    {* The various types of nodes in the Subversion filesystem.  }
  svn_node_kind_p = ^svn_node_kind_t;
  svn_node_kind_t = (
    { absent  }
    svn_node_none,
    { regular file  }
    svn_node_file,
    { directory  }
    svn_node_dir,
    { something's here, but we don't know what  }
    svn_node_unknown
  );
  
    {* About Special Files in Subversion
     *
     * Subversion denotes files that cannot be portably created or
     * modified as "special" files (svn_node_special).  It stores these
     * files in the repository as a plain text file with the svn:special
     * property set.  The file contents contain: a platform-specific type
     * string, a space character, then any information necessary to create
     * the file on a supported platform.  For example, if a symbolic link
     * were being represented, the repository file would have the
     * following contents:
     *
     * "link /path/to/link/target"
     *
     * Where 'link' is the identifier string showing that this special
     * file should be a symbolic link and '/path/to/link/target' is the
     * destination of the symbolic link.
     *
     * Special files are stored in the text-base exactly as they are
     * stored in the repository.  The platform specific files are created
     * in the working copy at EOL/keyword translation time using
     * svn_subst_copy_and_translate2().  If the current platform does not
     * support a specific special file type, the file is copied into the
     * working copy as it is seen in the repository.  Because of this,
     * users of other platforms can still view and modify the special
     * files, even if they do not have their unique properties.
     *
     * New types of special files can be added by:
     *  1. Implementing a platform-dependent routine to create a uniquely
     *     named special file and one to read the special file in
     *     libsvn_subr/io.c.
     *  2. Creating a new textual name similar to
     *     SVN_SUBST__SPECIAL_LINK_STR in libsvn_subr/subst.c.
     *  3. Handling the translation/detranslation case for the new type in
     *     create_special_file and detranslate_special_file, using the
     *     routines from 1.
      }
    {* A revision number.  }
  svn_revnum_p = ^svn_revnum_t;
  svn_revnum_t = longint;
    {* In printf()-style functions, format revision numbers using this.
     * Do not use this macro within the Subversion project source code, because
     * the language translation tools have trouble parsing it.  }

const
  SVN_REVNUM_T_FMT = 'ld';

type
  {* The size of a file in the Subversion FS.  }
  svn_filesize_t = apr_int64_t;

const
    {* The 'official' invalid file size constant.  }
    { was #define dname def_expr }
  SVN_INVALID_FILESIZE: svn_filesize_t = -1;

type
  {* YABT:  Yet Another Boolean Type  }
  svn_boolean_p = ^svn_boolean_t;
  svn_boolean_t = longbool;

type
  svn_recurse_kind = (
    svn_nonrecursive := 1,
    svn_recursive
  );

    {*
     * It is sometimes convenient to indicate which parts of an @c svn_dirent_t
     * object you are actually interested in, so that calculating and sending
     * the data corresponding to the other fields can be avoided.  These values
     * can be used for that purpose.
     *
     * @defgroup svn_dirent_fields dirent fields
     * @
      }
    {* An indication that you are interested in the @c kind field  }

const
  SVN_DIRENT_KIND = $00001;
    {* An indication that you are interested in the @c size field  }
  SVN_DIRENT_SIZE = $00002;
    {* An indication that you are interested in the @c has_props field  }
  SVN_DIRENT_HAS_PROPS = $00004;
    {* An indication that you are interested in the @c created_rev field  }
  SVN_DIRENT_CREATED_REV = $00008;
    {* An indication that you are interested in the @c time field  }
  SVN_DIRENT_TIME = $00010;
    {* An indication that you are interested in the @c last_author field  }
  SVN_DIRENT_LAST_AUTHOR = $00020;
    {* A combination of all the dirent fields  }
    { was #define dname def_expr }
  SVN_DIRENT_ALL: apr_uint32_t = $FFFFFFFF;
        { return type might be wrong }

type
    {* @  }
    {* A general subversion directory entry.  }
  psvn_dirent_t = ^svn_dirent_t;
  svn_dirent_p = ^svn_dirent_t;
  svn_dirent_t = record
    {* node kind  }
    kind: svn_node_kind_t;
    {* length of file text, or 0 for directories  }
    size: svn_filesize_t;
    {* does the node have props?  }
    has_props: svn_boolean_t;
    {* last rev in which this node changed  }
    created_rev: svn_revnum_t;
    {* time of created_rev (mod-time)  }
    time: apr_time_t;
    {* author of created_rev  }
    last_author: pchar;
    { IMPORTANT: If you extend this struct, check svn_dirent_dup().  }
  end;
    {* Return a deep copy of @a dirent, allocated in @a pool.
     *
     * @since New in 1.4.
      }

    function svn_dirent_dup(dirent: svn_dirent_p; pool: apr_pool_p): svn_dirent_p; cdecl; external libsvn_subr name 'svn_dirent_dup';

    {* Keyword substitution.
     *
     * All the keywords Subversion recognizes.
     * 
     * Note that there is a better, more general proposal out there, which
     * would take care of both internationalization issues and custom
     * keywords (e.g., $NetBSD$).  See
     * 
     *<pre>    http://subversion.tigris.org/servlets/ReadMsg?list=dev&msgNo=8921
     *    =====
     *    From: "Jonathan M. Manning" <jmanning@alisa-jon.net>
     *    To: dev@subversion.tigris.org
     *    Date: Fri, 14 Dec 2001 11:56:54 -0500
     *    Message-ID: <87970000.1008349014@bdldevel.bl.bdx.com>
     *    Subject: Re: keywords</pre>
     *
     * and Eric Gillespie's support of same:
     *
     *<pre>    http://subversion.tigris.org/servlets/ReadMsg?list=dev&msgNo=8757
     *    =====
     *    From: "Eric Gillespie, Jr." <epg@pretzelnet.org>
     *    To: dev@subversion.tigris.org
     *    Date: Wed, 12 Dec 2001 09:48:42 -0500
     *    Message-ID: <87k7vsebp1.fsf@vger.pretzelnet.org>
     *    Subject: Re: Customizable Keywords</pre>
     *
     * However, it is considerably more complex than the scheme below.
     * For now we're going with simplicity, hopefully the more general
     * solution can be done post-1.0.
     *
     * @defgroup svn_types_keywords keywords
     * @
      }
    {* The maximum size of an expanded or un-expanded keyword.  }

const
  SVN_KEYWORD_MAX_LEN = 255;
  {* The most recent revision in which this file was changed.  }
  SVN_KEYWORD_REVISION_LONG = 'LastChangedRevision';
  {* Short version of LastChangedRevision  }
  SVN_KEYWORD_REVISION_SHORT = 'Rev';
  {* Medium version of LastChangedRevision, matching the one CVS uses.
   * @since New in 1.1.  }
  SVN_KEYWORD_REVISION_MEDIUM = 'Revision';
  {* The most recent date (repository time) when this file was changed.  }
  SVN_KEYWORD_DATE_LONG = 'LastChangedDate';
  {* Short version of LastChangedDate  }
  SVN_KEYWORD_DATE_SHORT = 'Date';
  {* Who most recently committed to this file.  }
  SVN_KEYWORD_AUTHOR_LONG = 'LastChangedBy';
  {* Short version of LastChangedBy  }
  SVN_KEYWORD_AUTHOR_SHORT = 'Author'; 
  {* The URL for the head revision of this file.  }
  SVN_KEYWORD_URL_LONG = 'HeadURL';
  {* Short version of HeadURL  }
  SVN_KEYWORD_URL_SHORT = 'URL';
  {* A compressed combination of the other four keywords.  }
  SVN_KEYWORD_ID = 'Id';

type
    {* @  }
    {* All information about a commit.
     *
     * @note Objects of this type should always be created using the
     * svn_create_commit_info() function.
     *
     * @since New in 1.3.
      }
  svn_commit_info_p = ^svn_commit_info_t;
  svn_commit_info_t = record
    {* just-committed revision.  }
    revision: svn_revnum_t;
    {* server-side date of the commit.  }
    date: pchar;
    {* author of the commit.  }
    author: pchar;
    {* error message from post-commit hook, or NULL.  }
    post_commit_err: pchar;
  end;

    {*
     * Allocate an object of type @c svn_commit_info_t in @a pool and
     * return it.
     *
     * The @c revision field of the new struct is set to @c
     * SVN_INVALID_REVNUM.  All other fields are initialized to @c NULL.
     *
     * @note Any object of the type @c svn_commit_info_t should
     * be created using this function.
     * This is to provide for extending the svn_commit_info_t in
     * the future.
     *
     * @since New in 1.3.
      }

    function svn_create_commit_info(pool:apr_pool_p):svn_commit_info_p; cdecl; external libsvn_subr name 'svn_create_commit_info';

    {*
     * Return a deep copy @a src_commit_info allocated in @a pool.
     *
     * @since New in 1.4.
      }
    function svn_commit_info_dup(src_commit_info:svn_commit_info_p; pool:apr_pool_p):svn_commit_info_p; cdecl; external libsvn_subr name 'svn_commit_info_dup';

type
  {* A structure to represent a path that changed for a log entry.  }
  svn_log_changed_path_p = ^svn_log_changed_path_t;
  svn_log_changed_path_t = record
    {* 'A'dd, 'D'elete, 'R'eplace, 'M'odify  }
    action: char;
    {* Source path of copy (if any).  }
    copyfrom_path: pchar;
    {* Source revision of copy (if any).  }
    copyfrom_rev: svn_revnum_t;
  end;
    {*
     * Return a deep copy of @a changed_path, allocated in @a pool.
     *
     * @since New in 1.3.
      }
    function svn_log_changed_path_dup(changed_path:svn_log_changed_path_p; pool:apr_pool_p): svn_log_changed_path_p; cdecl; external libsvn_subr name 'svn_log_changed_path_dup';

    {* The callback invoked by log message loopers, such as
     * @c svn_ra_plugin_t.get_log() and svn_repos_get_logs().
     *
     * This function is invoked once on each log message, in the order
     * determined by the caller (see above-mentioned functions).
     *
     * @a baton, @a revision, @a author, @a date, and @a message are what you 
     * think they are.  Any of @a author, @a date, or @a message may be @c NULL.
     *
     * If @a date is neither null nor the empty string, it was generated by
     * svn_time_to_cstring() and can be converted to @c apr_time_t with
     * svn_time_from_cstring().
     *
     * If @a changed_paths is non-@c NULL, then it contains as keys every path
     * committed in @a revision; the values are (@c svn_log_changed_path_t *) 
     * structures.
     *
     * ### The only reason @a changed_paths is not qualified with `const' is
     * that we usually want to loop over it, and apr_hash_first() doesn't
     * take a const hash, for various reasons.  I'm not sure that those
     * "various reasons" are actually even relevant anymore, and if
     * they're not, it might be nice to change apr_hash_first() so
     * read-only uses of hashes can be protected via the type system.
     *
     * Use @a pool for all allocation.  (If the caller is iterating over log
     * messages, invoking this receiver on each, we recommend the standard
     * pool loop recipe: create a subpool, pass it as @a pool to each call,
     * clear it after each iteration, destroy it after the loop is done.)
      }
type
    { use svn_time_from_cstring() if need apr_time_t  }
  svn_log_message_receiver_t = function (
    baton: pointer;
    changed_paths: apr_hash_p;
    revision: svn_revnum_t;
    author: pchar;
    date: pchar;
    message: pchar;
    pool: apr_pool_p
  ): svn_error_p; cdecl;

    {* Callback function type for commits.
     *
     * When a commit succeeds, an instance of this is invoked with the
     * @a commit_info, along with the @a baton closure.
     * @a pool can be used for temporary allocations.
     *
     * @since New in 1.4.
      }
  svn_commit_callback2_t = function (
    commit_info: svn_commit_info_p;
    baton: pointer;
    pool: apr_pool_p
  ): svn_error_p; cdecl;

    {* Same as @c svn_commit_callback2_t, but uses individual
     * data elements instead of the @c svn_commit_info_t structure
     *
     * @deprecated Provided for backward compatibility with the 1.3 API.
      }
  svn_commit_callback_t = function (
    new_revision: svn_revnum_t;
    date: pchar;
    author: pchar;
    baton: pointer
  ): svn_error_p; cdecl;

    {* Return, in @a *callback2 and @a *callback2_baton a function/baton that
     * will call @a callback/@a callback_baton, allocating the @a *callback2_baton
     * in @a pool.
     *
     * @note This is used by compatibility wrappers, which exist in more than
     * Subversion core library.
     *
     * @since New in 1.4.
      }

    procedure svn_compat_wrap_commit_callback(out callback2: svn_commit_callback2_t; out callback2_baton: pointer; callback: svn_commit_callback_t; callback_baton: pointer; pool: apr_pool_p); cdecl; external libsvn_subr name 'svn_compat_wrap_commit_callback';

const
    {* A buffer size that may be used when processing a stream of data.
     *
     * @note We don't use this constant any longer, since it is considered to be
     * unnecessarily large.
     *
     * @deprecated Provided for backwards compatibility with the 1.3 API.
      }
  SVN_STREAM_CHUNK_SIZE = 102400;
  
    {
     * The maximum amount we (ideally) hold in memory at a time when
     * processing a stream of data.
     *
     * For example, when copying data from one stream to another, do it in
     * blocks of this size.
     *
     * NOTE: This is an internal macro, put here for convenience.
     * No public API may depend on the particular value of this macro.
      }

const
  SVN__STREAM_CHUNK_SIZE = 16384;       

    {* The maximum amount we can ever hold in memory.  }
    { FIXME: Should this be the same as SVN_STREAM_CHUNK_SIZE?  }
    { was #define dname def_expr }
    //#define SVN_MAX_OBJECT_SIZE (((apr_size_t) -1) / 2)
        { return type might be wrong }

    { ### Note: despite being about mime-TYPES, these probably don't
     * ### belong in svn_types.h.  However, no other header is more
     * ### appropriate, and didn't feel like creating svn_validate.h for
     * ### so little.
      }
    {* Validate @a mime_type.
     *
     * If @a mime_type does not contain a "/", or ends with non-alphanumeric
     * data, return @c SVN_ERR_BAD_MIME_TYPE, else return success.
     * 
     * Use @a pool only to find error allocation.
     *
     * Goal: to match both "foo/bar" and "foo/bar; charset=blah", without
     * being too strict about it, but to disallow mime types that have
     * quotes, newlines, or other garbage on the end, such as might be
     * unsafe in an HTTP header.
      }
  function svn_mime_type_validate(mime_type: pchar; pool: apr_pool_p): svn_error_p; cdecl; external libsvn_subr name 'svn_mime_type_validate';

    {* Return false iff @a mime_type is a textual type.
     *
     * All mime types that start with "text/" are textual, plus some special 
     * cases (for example, "image/x-xbitmap").
      }
  function svn_mime_type_is_binary(mime_type:pchar):svn_boolean_t; cdecl; external libsvn_subr name 'svn_mime_type_is_binary';

    {* A user defined callback that subversion will call with a user defined 
     * baton to see if the current operation should be continued.  If the operation 
     * should continue, the function should return @c SVN_NO_ERROR, if not, it 
     * should return @c SVN_ERR_CANCELLED.
      }

type
  svn_cancel_func_t = function (cancel_baton: pointer): svn_error_p; cdecl;

    {*
     * A lock object, for client & server to share.
     *
     * A lock represents the exclusive right to add, delete, or modify a
     * path.  A lock is created in a repository, wholly controlled by the
     * repository.  A "lock-token" is the lock's UUID, and can be used to
     * learn more about a lock's fields, and or/make use of the lock.
     * Because a lock is immutable, a client is free to not only cache the
     * lock-token, but the lock's fields too, for convenience.
     *
     * Note that the 'is_dav_comment' field is wholly ignored by every
     * library except for mod_dav_svn.  The field isn't even marshalled
     * over the network to the client.  Assuming lock structures are
     * created with apr_pcalloc(), a default value of 0 is universally safe.
     *
     * @note in the current implementation, only files are lockable.
     *
     * @since New in 1.2.
      }
    {*< the path this lock applies to  }
    {*< unique URI representing lock  }
    {*< the username which owns the lock  }
    {*< (optional) description of lock   }
    {*< was comment made by generic DAV client?  }
    {*< when lock was made  }
    {*< (optional) when lock will expire;
                                          If value is 0, lock will never expire.  }

  svn_lock_p = ^svn_lock_t;
  svn_lock_t = record
    path: pchar;
    token: pchar;
    owner: pchar;
    comment: pchar;
    is_dav_comment: svn_boolean_t;
    creation_date: apr_time_t;
    expiration_date: apr_time_t;
  end;
    {*
     * Returns an @c svn_lock_t, allocated in @a pool with all fields initialized
     * to null values.
     *
     * @note To allow for extending the @c svn_lock_t structure in the future
     * releases, this function should always be used to allocate the structure.
     *
     * @since New in 1.2.
      }

    function svn_lock_create(pool:apr_pool_p): svn_lock_p; cdecl; external libsvn_subr name 'svn_lock_create';

    {*
     * Return a deep copy of @a lock, allocated in @a pool.
     *
     * @since New in 1.2.
      }
    function svn_lock_dup(lock:svn_lock_p; pool:apr_pool_p): svn_lock_p; cdecl; external libsvn_subr name 'svn_lock_dup';

    {*
     * Return a formatted Universal Unique IDentifier (UUID) string.
     *
     * @since New in 1.4.
      }
    function svn_uuid_generate(pool:apr_pool_p): pchar; cdecl; external libsvn_subr name 'svn_uuid_generate';
