unit mxrpc;

{$IFDEF FPC}
 {$MODE DELPHI}
{$ENDIF}
{$ASSERTIONS ON}

interface
uses SysUtils, Classes, curl_h, mxvalues, libxml2, mxwriter;

type
  TMXRPC_State = (readFault, readValue, readRecord, readArray);
  TMXRPC = class
    private
      FCURL: pCURL;
      FGateURL: UTF8String;
      FRequest: TMXArrayValue;
      FResponse: TMXValue;
      FContent: TStringStream;
      FCaptureText: Boolean;
      FText: UTF8String;
      FIsFault: Boolean;
      FFaultCode: Integer;
      FFaultString: UTF8String;
      FState: TMXRPC_State;
      FStateStack: array of TMXRPC_State;
      FMember: UTF8String;
      FRecord: TMXRecordValue;
      FArray: TMXArrayValue;
      FQueryCount: Integer;
    protected
      procedure ParseResponse;
      procedure EncodeRequest(const Method: UTF8String);
      
      procedure CaptureText;
      function GetText: UTF8String;
      procedure pushState(State: TMXRPC_State);
      function popState: TMXRPC_State;
      procedure EndFault(LocalName: xmlCharPtr);
      procedure EndValue(LocalName: xmlCharPtr);
      procedure EndRecordMember(LocalName: xmlCharPtr);
      procedure EndArrayItem(LocalName: xmlCharPtr);
      //procedure OnCharacters(Sender: TObject; const ch: PSAXChar; AStart, ALength: Integer);
      //procedure OnIgnorableWhitespace(Sender: TObject; const ch: PSAXChar; AStart, ALength: Integer);
      //procedure OnStartElement(Sender: TObject; const NamespaceURI, LocalName, QName: SAXString; Atts: TSAXAttributes);
      //procedure OnEndElement(Sender: TObject; const NamespaceURI, LocalName, QName: SAXString);
    public
      constructor Create(const AGateURL: UTF8String);
      destructor Destroy; override;
      
      function Invoke(const Method: UTF8String): Boolean;
      
      property GateURL: UTF8String read FGateURL;
      property Request: TMXArrayValue read FRequest;
      property Response: TMXValue read FResponse;
      property IsFault: Boolean read FIsFault;
      property FaultCode: Integer read FFaultCode;
      property FaultString: UTF8String read FFaultString;
      property QueryCount: Integer read FQueryCount;
  end;

implementation

constructor TMXRPC.Create(const AGateURL: UTF8String);
begin
  FCURL := curl_easy_init();
  FGateURL := AGateURL;
  assert(FCURL <> nil, 'curl_easy_init() fault');
  FRequest := TMXArrayValue.Create;
  FResponse := TMXValue.Create;
  FContent := TStringStream.Create('');
end;

destructor TMXRPC.Destroy;
begin
  FContent.Free;
  FRequest.Free;
  FResponse.Free;
  curl_easy_cleanup(FCURL);
end;

function write_data(Buffer: Pointer; Size, nmemb: Cardinal; RPC: TMXRPC): Cardinal; cdecl;
begin
  Result := RPC.FContent.Write(Buffer^, Size * nmemb);
end;

function TMXRPC.Invoke(const Method: UTF8String): Boolean;
var
  code: CURLcode;
  req: AnsiString;
begin
  Inc(FQueryCount);
  code := curl_easy_setopt(FCURL, CURLOPT_URL, pchar(FGateURL));
  assert(code = CURLE_OK, 'curl_easy_setopt(CURLOPT_URL) fault');
  
  code := curl_easy_setopt(FCURL, CURLOPT_WRITEFUNCTION, @write_data);
  assert(code = CURLE_OK, 'curl_easy_setopt(CURLOPT_WRITEFUNCTION) fault');
  
  code := curl_easy_setopt(FCURL, CURLOPT_WRITEDATA, Pointer(Self));
  assert(code = CURLE_OK, 'curl_easy_setopt(CURLOPT_WRITEDATA) fault');
  
  code := curl_easy_setopt(FCURL, CURLOPT_POST, 1);
  assert(code = CURLE_OK, 'curl_easy_setopt(CURLOPT_POST) fault');
  
  EncodeRequest(method);
  req := FContent.DataString;
  FContent.Position := 0;
  FContent.Size := 0;
  
  code := curl_easy_setopt(FCURL, CURLOPT_POSTFIELDS, pchar(req));
  assert(code = CURLE_OK, 'curl_easy_setopt(CURLOPT_POSTFIELDSIZE) fault');
  
  code := curl_easy_setopt(FCURL, CURLOPT_POSTFIELDSIZE, length(req));
  assert(code = CURLE_OK, 'curl_easy_setopt(CURLOPT_POSTFIELDSIZE) fault');
  
  code := curl_easy_perform(FCURL);
  
  Result := code = CURLE_OK;
  assert(Result, 'curl_easy_perform() fault');
  
  if Result then
  begin
    ParseResponse;
  end;
end;

procedure TMXRPC.CaptureText;
begin
  FCaptureText := true;
  FText := '';
end;

function TMXRPC.GetText: UTF8String;
begin
  FCaptureText := false;
  Result := FText;
end;

procedure OnCharacters(ctx: Pointer; const ch: xmlCharPtr; len: Longint); cdecl;
var str: UTF8String;
begin
  with TMXRPC(ctx) do
    if FCaptureText and (len > 0) then
    begin
      SetString(str, ch, len);
      FText := FText + str;
    end; // if
end;

procedure OnStartElement(ctx: Pointer; const name: xmlCharPtr; const atts: xmlCharPtrPtr); cdecl;
begin
  with TMXRPC(ctx) do
  begin
    if (name = 'int') or (name = 'i4') or (name = 'double') or
      (name = 'boolean') or (name = 'name') or (name = 'string') then
    begin
      CaptureText;
    end
    else if name = 'fault' then
    begin
      pushState(FState);
      FState := readFault;
    end
    else if name = 'struct' then
    begin
      case FState of
      readFault: FRecord := Response.AsRecord;
      readValue: FRecord := Response.AsRecord;
      readRecord: FRecord := FRecord.AsRecord[FMember];
      readArray: FRecord := FArray.AddRecord;
      end;
      pushState(FState);
      FState := readRecord;
    end
    else if name = 'array' then
    begin
      case FState of
      readValue: FArray := Response.AsArray;
      readRecord: FArray := FRecord.AsArray[FMember];
      readArray: FArray := FArray.AddArray;
      end;
      pushState(FState);
      FState := readArray;
    end; // if
  end; // with
end;

procedure TMXRPC.pushState(State: TMXRPC_State);
var Len: Integer;
begin
  Len := Length(FStateStack);
  SetLength(FStateStack, Len + 1);
  FStateStack[Len] := State;
end;

function TMXRPC.popState: TMXRPC_State;
var Len: Integer;
begin
  Len := Length(FStateStack);
  Result := FStateStack[Len-1];
  SetLength(FStateStack, Len-1);
end;

procedure TMXRPC.EndFault(LocalName: xmlCharPtr);
begin
  if LocalName = 'fault' then
  begin
    FIsFault := true;
    FFaultCode := FRecord.AsInteger['faultCode'];
    FFaultString := FRecord.AsString['faultString'];
  end; // if
end;

procedure TMXRPC.EndValue(LocalName: xmlCharPtr);
var str: UTF8String;
begin
  if (LocalName = 'int') or (LocalName = 'i4') then
  begin
    Response.AsInteger := StrToInt(Trim(GetText));
  end
  else if LocalName = 'double' then
  begin
    Response.AsFloat := StrToFloat(Trim(GetText));
  end
  else if LocalName = 'boolean' then
  begin
    str := Trim(GetText);
    Response.AsBoolean := (str = '1') or (str = 'true');
  end
  else if LocalName = 'string' then
  begin
    Response.AsString := GetText;
  end; // if
end;

procedure TMXRPC.EndRecordMember(LocalName: xmlCharPtr);
var str: UTF8String;
begin
  if (LocalName = 'int') or (LocalName = 'i4') then
  begin
    FRecord.AsInteger[FMember] := StrToInt(Trim(GetText));
  end
  else if LocalName = 'double' then
  begin
    FRecord.AsFloat[FMember] := StrToFloat(Trim(GetText));
  end
  else if LocalName = 'boolean' then
  begin
    str := Trim(GetText);
    FRecord.AsBoolean[FMember] := (str = '1') or (str = 'true');
  end
  else if LocalName = 'string' then
  begin
    FRecord.AsString[FMember] := GetText;
  end
  else if LocalName = 'name' then
  begin
    FMember := Trim(GetText);
  end; // if
end;

procedure TMXRPC.EndArrayItem(LocalName: xmlCharPtr);
var str: UTF8String;
begin
  if (LocalName = 'int') or (LocalName = 'i4') then
  begin
    FArray.AddInteger(StrToInt(Trim(GetText)));
  end
  else if LocalName = 'double' then
  begin
    FArray.AddFloat(StrToFloat(Trim(GetText)));
  end
  else if LocalName = 'boolean' then
  begin
    str := Trim(GetText);
    FArray.AddBoolean( (str = '1') or (str = 'true') );
  end
  else if LocalName = 'string' then
  begin
    FArray.AddString(GetText);
  end; // if
end;

procedure OnEndElement(ctx: Pointer; const name: xmlCharPtr); cdecl;
begin
  with TMXRPC(ctx) do
  begin
    if (name = 'struct') or (name = 'array') then
    begin
      FState := popState;
    end; // if
    case FState of
    readFault: EndFault(name);
    readValue: EndValue(name);
    readRecord: EndRecordMember(name);
    readArray: EndArrayItem(name);
    end; // case
  end; // with
end;

procedure TMXRPC.ParseResponse;
var
  sax: xmlSAXHandler;
begin
  FState := readValue;
  FStateStack := nil;
  FIsFault := false;
  FFaultCode := 0;
  FFaultString := 'OK';
  FillChar(sax, SizeOf(sax), 0);
  sax.startElement := OnStartElement;
  sax.endElement := OnEndElement;
  sax.characters := OnCharacters;
  if xmlSAXUserParseMemory(@sax, Self, pchar(FContent.DataString), Length(FContent.DataString)) <> 0 then
  begin
    FIsFault := true;
    FFaultCode := -1;
    FFaultString := 'response parse fault';
    writeln(stderr, 'error: response parse fault');
    writeln(stderr, FContent.DataString);
  end; // if
end;

procedure EncodeValue(xml: TMXWriter; Value: TMXAbstractValue);
var i, count: Integer;
begin
  xml.Open('value');
  case Value.ValueType of
  mxIntegerValue: xml.WriteTag('int', Value.AsString);
  mxFloatValue: xml.WriteTag('double', FloatToStr(Value.AsFloat));
  mxBooleanValue: if Value.AsBoolean then xml.WriteTag('boolean', 'true') else xml.WriteTag('boolean', 'false');
  mxStringValue: xml.WriteTag('string', Value.AsString);
  mxRecordValue:
    begin
      xml.Open('struct');
      count := TMXRecordValue(Value).Count;
      for i := 0 to count-1 do
      begin
        xml.Open('member');
        xml.WriteTag('name', TMXRecordValue(Value).Names[i]);
        EncodeValue(xml, TMXRecordValue(Value).Fields[i]);
        xml.Close('member');
      end; // for i
      xml.Close('struct');
    end; // mxRecordValue
  mxArrayValue:
    begin
      xml.Open('array');
      xml.Open('data');
      count := TMXArrayValue(Value).Count;
      for i := 0 to count-1 do
      begin
        EncodeValue(xml, TMXArrayValue(Value).AsValue[i]);
      end; // for i
      xml.Close('data');
      xml.Close('array');
    end; // mxArrayValue
  else assert(false);
  end;
  xml.Close('value');
end;

procedure TMXRPC.EncodeRequest(const Method: UTF8String);
var
  i, len: Integer;
  xml: TMXWriter;
begin
  FContent.Position := 0;
  FContent.Size := 0;
  xml := TMXWriter.Create(FContent);
  try
    xml.Open('methodCall');
      xml.WriteTag('methodName', Method);
      xml.Open('params');
        len := Request.Count;
        for i := 0 to len-1 do
        begin
          xml.Open('param');
          EncodeValue(xml, Request.AsValue[i]);
          xml.Close('param');
        end; // for i
      xml.Close('params');
    xml.Close('methodCall');
  finally
    xml.Free;
  end;
end;

initialization

  assert(curl_global_init(CURL_GLOBAL_ALL) = CURLE_OK, 'curl_global_init() fault');

finalization

  curl_global_cleanup;

end.
