unit mxvalues;

{$IFDEF FPC}
 {$MODE DELPHI}
{$ENDIF}

interface
uses SysUtils, Classes, DateUtils;

type
  // типы значений
  TMXValueType = (
    mxNullValue,
    mxIntegerValue,
    mxFloatValue,
    mxBooleanValue,
    mxStringValue,
    mxDateTimeValue,
    mxBinaryValue,
    mxRecordValue,
    mxArrayValue
  );

const
  MXValueTypeName: array [ TMXValueType ] of AnsiString = (
    'NULL',
    'Integer',
    'Float',
    'Boolean',
    'String',
    'DateTime',
    'Binary',
    'Record',
    'Array'
  );
  
type
  TMXValue = class;
  TMXRecordValue = class;
  TMXArrayValue = class;
  
  TMXAbstractValue = class
    protected
      function GetValueType: TMXValueType; virtual; abstract;
      function GetInteger: Integer; virtual; abstract;
      function GetFloat: Extended; virtual; abstract;
      function GetBoolean: Boolean; virtual; abstract;
      function GetString: UTF8String; virtual; abstract;
      function GetDateTime: TDateTime; virtual; abstract;
    public
      property ValueType: TMXValueType read GetValueType;
      property AsInteger: Integer read GetInteger;
      property AsFloat: Extended read GetFloat;
      property AsBoolean: Boolean read GetBoolean;
      property AsString: UTF8String read GetString;
      property AsDateTime: TDateTime read GetDateTime;
  end;
  
  TMXRecordValue = class (TMXAbstractValue)
    private
      FList: TStringList;
      
      function GetValueType: TMXValueType; override;
      function GetInteger: Integer; override;
      function GetFloat: Extended; override;
      function GetBoolean: Boolean; override;
      function GetString: UTF8String; override;
      function GetDateTime: TDateTime; override;
      
      function GetCount: Integer;
      function GetItemValueType(const Name: AnsiString): TMXValueType;
      function GetItemInteger(const Name: AnsiString): Integer;
      procedure SetItemInteger(const Name: AnsiString; AValue: Integer);
      function GetItemFloat(const Name: AnsiString): Extended;
      procedure SetItemFloat(const Name: AnsiString; AValue: Extended);
      function GetItemBoolean(const Name: AnsiString): Boolean;
      procedure SetItemBoolean(const Name: AnsiString; AValue: Boolean);
      function GetItemString(const Name: AnsiString): UTF8String;
      procedure SetItemString(const Name: AnsiString; const AValue: UTF8String);
      function GetItemDateTime(const Name: AnsiString): TDateTime;
      procedure SetItemDateTime(const Name: AnsiString; AValue: TDateTime);
      function GetItemRecord(const Name: AnsiString): TMXRecordValue;
      function GetItemArray(const Name: AnsiString): TMXArrayValue;
      function GetItemName(Idx: Integer): AnsiString;
      function GetItem(Idx: Integer): TMXAbstractValue;
    public
      constructor Create;
      destructor Destroy; override;
      
      procedure Clear;
      
      function Exists(const Name: AnsiString): Boolean;
      procedure Remove(const Name: AnsiString);
      
      property Count: Integer read GetCount;
      property ValueType[const Name: AnsiString]: TMXValueType read GetItemValueType;
      property AsInteger[const Name: AnsiString]: Integer read GetItemInteger write SetItemInteger;
      property AsFloat[const Name: AnsiString]: Extended read GetItemFloat write SetItemFloat;
      property AsBoolean[const Name: AnsiString]: Boolean read GetItemBoolean write SetItemBoolean;
      property AsString[const Name: AnsiString]: UTF8String read GetItemString write SetItemString;
      property AsDateTime[const Name: AnsiString]: TDateTime read GetItemDateTime write SetItemDateTime;
      property AsRecord[const Name: AnsiString]: TMXRecordValue read GetItemRecord;
      property AsArray[const Name: AnsiString]: TMXArrayValue read GetItemArray;
      property Names[Idx: Integer]: AnsiString read GetItemName;
      property Fields[Idx: Integer]: TMXAbstractValue read GetItem;
  end;
  
  TMXArrayValue = class (TMXAbstractValue)
    private
      FList: TList;
      
      function GetValueType: TMXValueType; override;
      function GetInteger: Integer; override;
      function GetFloat: Extended; override;
      function GetBoolean: Boolean; override;
      function GetString: UTF8String; override;
      function GetDateTime: TDateTime; override;
      
      function GetCount: Integer;
      function GetItemValueType(Idx: Integer): TMXValueType;
      function GetItemInteger(Idx: Integer): Integer;
      procedure SetItemInteger(Idx: Integer; AValue: Integer);
      function GetItemFloat(Idx: Integer): Extended;
      procedure SetItemFloat(Idx: Integer; AValue: Extended);
      function GetItemBoolean(Idx: Integer): Boolean;
      procedure SetItemBoolean(Idx: Integer; AValue: Boolean);
      function GetItemString(Idx: Integer): UTF8String;
      procedure SetItemString(Idx: Integer; const AValue: UTF8String);
      function GetItemDateTime(Idx: Integer): TDateTime;
      procedure SetItemDateTime(Idx: Integer; AValue: TDateTime);
      function GetItemRecord(Idx: Integer): TMXRecordValue;
      function GetItemArray(Idx: Integer): TMXArrayValue;
      function GetItem(Idx: Integer): TMXAbstractValue;
    public
      constructor Create;
      destructor Destroy; override;
      
      procedure Clear;
      
      procedure AddInteger(AValue: Integer);
      procedure AddFloat(AValue: Extended);
      procedure AddBoolean(AValue: Boolean);
      procedure AddString(const AValue: UTF8String);
      procedure AddDateTime(AValue: TDateTime);
      function AddRecord: TMXRecordValue;
      function AddArray: TMXArrayValue;
      
      procedure Delete(Idx: Integer);
      
      property Count: Integer read GetCount;
      property ValueType[Idx: Integer]: TMXValueType read GetItemValueType;
      property AsInteger[Idx: Integer]: Integer read GetItemInteger write SetItemInteger;
      property AsFloat[Idx: Integer]: Extended read GetItemFloat write SetItemFloat;
      property AsBoolean[Idx: Integer]: Boolean read GetItemBoolean write SetItemBoolean;
      property AsString[Idx: Integer]: UTF8String read GetItemString write SetItemString;
      property AsDateTime[Idx: Integer]: TDateTime read GetItemDateTime write SetItemDateTime;
      property AsRecord[Idx: Integer]: TMXRecordValue read GetItemRecord;
      property AsArray[Idx: Integer]: TMXArrayValue read GetItemArray;
      property AsValue[Idx: Integer]: TMXAbstractValue read GetItem;
  end;
  
  TMXValue = class
    private
      FValue: TMXAbstractValue;
      
    protected
      function GetValueType: TMXValueType;
      function GetInteger: Integer;
      procedure SetInteger(AValue: Integer);
      function GetFloat: Extended;
      procedure SetFloat(AValue: Extended);
      function GetBoolean: Boolean;
      procedure SetBoolean(AValue: Boolean);
      function GetString: UTF8String;
      procedure SetString(const AValue: UTF8String);
      function GetDateTime: TDateTime;
      procedure SetDateTime(AValue: TDateTime);
      function GetRecord: TMXRecordValue;
      function GetArray: TMXArrayValue;
    public
      constructor Create;
      destructor Destroy; override;
      
      procedure SetNull;
      
      property ValueType: TMXValueType read GetValueType;
      property AsInteger: Integer read GetInteger write SetInteger;
      property AsFloat: Extended read GetFloat write SetFloat;
      property AsBoolean: Boolean read GetBoolean write SetBoolean;
      property AsString: UTF8String read GetString write SetString;
      property AsDateTime: TDateTime read GetDateTime write SetDateTime;
      property AsRecord: TMXRecordValue read GetRecord;
      property AsArray: TMXArrayValue read GetArray;
  end;

implementation

type
  TMXIntegerValue = class (TMXAbstractValue)
    private
      FValue: Integer;
    public
      constructor Create(AValue: Integer);
      function GetValueType: TMXValueType; override;
      function GetInteger: Integer; override;
      function GetFloat: Extended; override;
      function GetBoolean: Boolean; override;
      function GetString: UTF8String; override;
      function GetDateTime: TDateTime; override;
  end;
  
  TMXFloatValue = class (TMXAbstractValue)
    private
      FValue: Extended;
    public
      constructor Create(AValue: Extended);
      function GetValueType: TMXValueType; override;
      function GetInteger: Integer; override;
      function GetFloat: Extended; override;
      function GetBoolean: Boolean; override;
      function GetString: UTF8String; override;
      function GetDateTime: TDateTime; override;
  end;
  
  TMXBooleanValue = class (TMXAbstractValue)
    private
      FValue: Boolean;
    public
      constructor Create(AValue: Boolean);
      function GetValueType: TMXValueType; override;
      function GetInteger: Integer; override;
      function GetFloat: Extended; override;
      function GetBoolean: Boolean; override;
      function GetString: UTF8String; override;
      function GetDateTime: TDateTime; override;
  end;
  
  TMXStringValue = class (TMXAbstractValue)
    private
      FValue: UTF8String;
    public
      constructor Create(AValue: UTF8String);
      function GetValueType: TMXValueType; override;
      function GetInteger: Integer; override;
      function GetFloat: Extended; override;
      function GetBoolean: Boolean; override;
      function GetString: UTF8String; override;
      function GetDateTime: TDateTime; override;
  end;
  
  TMXDateTimeValue = class (TMXAbstractValue)
    private
      FValue: TDateTime;
    public
      constructor Create(AValue: TDateTime);
      function GetValueType: TMXValueType; override;
      function GetInteger: Integer; override;
      function GetFloat: Extended; override;
      function GetBoolean: Boolean; override;
      function GetString: UTF8String; override;
      function GetDateTime: TDateTime; override;
  end;

constructor TMXIntegerValue.Create(AValue: Integer);
begin
  FValue := AValue;
end;

function TMXIntegerValue.GetValueType: TMXValueType;
begin
  Result := mxIntegerValue;
end;

function TMXIntegerValue.GetInteger: Integer;
begin
  Result := FValue;
end;

function TMXIntegerValue.GetFloat: Extended;
begin
  Result := FValue;
end;

function TMXIntegerValue.GetBoolean: Boolean;
begin
  Result := FValue <> 0;
end;

function TMXIntegerValue.GetString: UTF8String;
begin
  str(FValue, Result);
end;

function TMXIntegerValue.GetDateTime: TDateTime;
begin
  Result := UnixToDateTime(FValue);
end;

constructor TMXFloatValue.Create(AValue: Extended);
begin
  FValue := AValue;
end;

function TMXFloatValue.GetValueType: TMXValueType;
begin
  Result := mxFloatValue;
end;

function TMXFloatValue.GetInteger: Integer;
begin
  Result := Round(FValue);
end;

function TMXFloatValue.GetFloat: Extended;
begin
  Result := FValue;
end;

function TMXFloatValue.GetBoolean: Boolean;
begin
  Result := FValue <> 0;
end;

function TMXFloatValue.GetString: UTF8String;
begin
  Result := FloatToStr(FValue);
end;

function TMXFloatValue.GetDateTime: TDateTime;
begin
  Result := FValue;
end;

constructor TMXBooleanValue.Create(AValue: Boolean);
begin
  FValue := AValue;
end;

function TMXBooleanValue.GetValueType: TMXValueType;
begin
  Result := mxBooleanValue;
end;

function TMXBooleanValue.GetInteger: Integer;
begin
  if FValue then Result := 1
  else Result := 0;
end;

function TMXBooleanValue.GetFloat: Extended;
begin
  if FValue then Result := 1
  else Result := 0;
end;

function TMXBooleanValue.GetBoolean: Boolean;
begin
  Result := FValue;
end;

function TMXBooleanValue.GetString: UTF8String;
begin
  if FValue then Result := '1'
  else Result := '';
end;

function TMXBooleanValue.GetDateTime: TDateTime;
begin
  Result := 0;
end;

constructor TMXStringValue.Create(AValue: UTF8String);
begin
  FValue := AValue;
end;

function TMXStringValue.GetValueType: TMXValueType;
begin
  Result := mxStringValue;
end;

function TMXStringValue.GetInteger: Integer;
begin
  Result := StrToInt(FValue);
end;

function TMXStringValue.GetFloat: Extended;
begin
  Result := StrToFloat(FValue);
end;

function TMXStringValue.GetBoolean: Boolean;
begin
  Result := FValue <> '';
end;

function TMXStringValue.GetString: UTF8String;
begin
  Result := FValue;
end;

function TMXStringValue.GetDateTime: TDateTime;
begin
  Result := StrToDateTime(FValue);
end;

constructor TMXDateTimeValue.Create(AValue: TDateTime);
begin
  FValue := AValue;
end;

function TMXDateTimeValue.GetValueType: TMXValueType;
begin
  Result := mxDateTimeValue;
end;

function TMXDateTimeValue.GetInteger: Integer;
begin
  Result := DateTimeToUnix(FValue);
end;

function TMXDateTimeValue.GetFloat: Extended;
begin
  Result := FValue;
end;

function TMXDateTimeValue.GetString: UTF8String;
begin
  Result := DateTimeToStr(FValue);
end;

function TMXDateTimeValue.GetBoolean: Boolean;
begin
  Result := FValue <> 0;
end;

function TMXDateTimeValue.GetDateTime: TDateTime;
begin
  Result := FValue;
end;

constructor TMXRecordValue.Create;
begin
  FList := TStringList.Create;
  FList.CaseSensitive := true;
end;

destructor TMXRecordValue.Destroy;
begin
  Clear;
  FList.Free;
end;

procedure TMXRecordValue.Clear;
var i, len: Integer;
begin
  len := FList.Count;
  for i := 0 to len-1 do
  begin
    FList.Objects[i].Destroy;
  end; // for i
  FList.Clear;
end;

function TMXRecordValue.GetValueType: TMXValueType;
begin
  Result := mxRecordValue;
end;

function TMXRecordValue.GetInteger: Integer;
begin
  Result := 1;
end;

function TMXRecordValue.GetFloat: Extended;
begin
  Result := 1;
end;

function TMXRecordValue.GetBoolean: Boolean;
begin
  Result := true;
end;

function TMXRecordValue.GetString: UTF8String;
begin
  Result := 'Record';
end;

function TMXRecordValue.GetDateTime: TDateTime;
begin
  Result := 0;
end;

function TMXRecordValue.GetCount: Integer;
begin
  Result := FList.Count;
end;

function TMXRecordValue.GetItemValueType(const Name: AnsiString): TMXValueType;
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then Result := TMXAbstractValue(FList.Objects[Idx]).GetValueType
  else Result := mxNullValue;
end;

function TMXRecordValue.GetItemInteger(const Name: AnsiString): Integer;
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then Result := TMXAbstractValue(FList.Objects[Idx]).GetInteger
  else Result := 0;
end;

procedure TMXRecordValue.SetItemInteger(const Name: AnsiString; AValue: Integer);
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then
  begin
    FList.Objects[Idx].Destroy;
    FList.Objects[Idx] := TMXIntegerValue.Create(AValue);
  end
  else
  begin
    FList.AddObject(Name, TMXIntegerValue.Create(AValue));
  end; // if
end;

function TMXRecordValue.GetItemFloat(const Name: AnsiString): Extended;
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then Result := TMXAbstractValue(FList.Objects[Idx]).GetFloat
  else Result := 0;
end;

procedure TMXRecordValue.SetItemFloat(const Name: AnsiString; AValue: Extended);
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then
  begin
    FList.Objects[Idx].Destroy;
    FList.Objects[Idx] := TMXFloatValue.Create(AValue);
  end
  else
  begin
    FList.AddObject(Name, TMXFloatValue.Create(AValue));
  end; // if
end;

function TMXRecordValue.GetItemBoolean(const Name: AnsiString): Boolean;
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then Result := TMXAbstractValue(FList.Objects[Idx]).GetBoolean
  else Result := false;
end;

procedure TMXRecordValue.SetItemBoolean(const Name: AnsiString; AValue: Boolean);
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then
  begin
    FList.Objects[Idx].Destroy;
    FList.Objects[Idx] := TMXBooleanValue.Create(AValue);
  end
  else
  begin
    FList.AddObject(Name, TMXBooleanValue.Create(AValue));
  end; // if
end;

function TMXRecordValue.GetItemString(const Name: AnsiString): UTF8String;
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then Result := TMXAbstractValue(FList.Objects[Idx]).GetString
  else Result := '';
end;

procedure TMXRecordValue.SetItemString(const Name: AnsiString; const AValue: UTF8String);
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then
  begin
    FList.Objects[Idx].Destroy;
    FList.Objects[Idx] := TMXStringValue.Create(AValue);
  end
  else
  begin
    FList.AddObject(Name, TMXStringValue.Create(AValue));
  end; // if
end;

function TMXRecordValue.GetItemDateTime(const Name: AnsiString): TDateTime;
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then Result := TMXAbstractValue(FList.Objects[Idx]).GetDateTime
  else Result := 0;
end;

procedure TMXRecordValue.SetItemDateTime(const Name: AnsiString; AValue: TDateTime);
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then
  begin
    FList.Objects[Idx].Destroy;
    FList.Objects[Idx] := TMXDateTimeValue.Create(AValue);
  end
  else
  begin
    FList.AddObject(Name, TMXDateTimeValue.Create(AValue));
  end; // if
end;

function TMXRecordValue.GetItemRecord(const Name: AnsiString): TMXRecordValue;
var
  Idx: Integer;
  Value: TMXAbstractValue;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then
  begin
    Value := TMXAbstractValue(FList.Objects[Idx]);
    if Value.GetValueType <> mxRecordValue then
    begin
      Value.Destroy;
      Value := TMXRecordValue.Create;
      FList.Objects[Idx] := Value;
    end; // if
    Result := TMXRecordValue(Value);
  end
  else
  begin
    Result := TMXRecordValue.Create;
    FList.AddObject(Name, Result);
  end; // if
end;

function TMXRecordValue.GetItemArray(const Name: AnsiString): TMXArrayValue;
var
  Idx: Integer;
  Value: TMXAbstractValue;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then
  begin
    Value := TMXAbstractValue(FList.Objects[Idx]);
    if Value.GetValueType <> mxArrayValue then
    begin
      Value.Destroy;
      Value := TMXArrayValue.Create;
      FList.Objects[Idx] := Value;
    end; // if
    Result := TMXArrayValue(Value);
  end
  else
  begin
    Result := TMXArrayValue.Create;
    FList.AddObject(Name, Result);
  end; // if
end;

function TMXRecordValue.GetItemName(Idx: Integer): AnsiString;
begin
  Result := FList[Idx];
end;

function TMXRecordValue.GetItem(Idx: Integer): TMXAbstractValue;
begin
  Result := TMXAbstractValue(FList.Objects[Idx]);
end;

function TMXRecordValue.Exists(const Name: AnsiString): Boolean;
begin
  Result := FList.IndexOf(Name) >= 0;
end;

procedure TMXRecordValue.Remove(const Name: AnsiString);
var Idx: Integer;
begin
  Idx := FList.IndexOf(Name);
  if Idx >= 0 then
  begin
    FList.Objects[Idx].Destroy;
    FList.Delete(Idx);
  end; // if
end;

constructor TMXArrayValue.Create;
begin
  FList := TList.Create;
end;

destructor TMXArrayValue.Destroy;
begin
  Clear;
  FList.Free;
end;

procedure TMXArrayValue.Clear;
var i, len: Integer;
begin
  len := FList.Count;
  for i := 0 to len-1 do
  begin
    TMXAbstractValue(FList[i]).Destroy;
  end; // for i
  FList.Clear;
end;

function TMXArrayValue.GetValueType: TMXValueType;
begin
  Result := mxArrayValue;
end;

function TMXArrayValue.GetInteger: Integer;
begin
  Result := 1;
end;

function TMXArrayValue.GetFloat: Extended;
begin
  Result := 1;
end;

function TMXArrayValue.GetBoolean: Boolean;
begin
  Result := true;
end;

function TMXArrayValue.GetString: UTF8String;
begin
  Result := 'Array';
end;

function TMXArrayValue.GetDateTime: TDateTime;
begin
  Result := 0;
end;

function TMXArrayValue.GetCount: Integer;
begin
  Result := FList.Count;
end;

function TMXArrayValue.GetItemValueType(Idx: Integer): TMXValueType;
begin
  Result := TMXAbstractValue(FList[Idx]).GetValueType;
end;

function TMXArrayValue.GetItemInteger(Idx: Integer): Integer;
begin
  Result := TMXAbstractValue(FList[Idx]).GetInteger;
end;

procedure TMXArrayValue.SetItemInteger(Idx: Integer; AValue: Integer);
begin
  TMXAbstractValue(FList[Idx]).Destroy;
  FList[Idx] := TMXIntegerValue.Create(AValue);
end;

function TMXArrayValue.GetItemFloat(Idx: Integer): Extended;
begin
  Result := TMXAbstractValue(FList[Idx]).GetFloat;
end;

procedure TMXArrayValue.SetItemFloat(Idx: Integer; AValue: Extended);
begin
  TMXAbstractValue(FList[Idx]).Destroy;
  FList[Idx] := TMXFloatValue.Create(AValue);
end;

function TMXArrayValue.GetItemBoolean(Idx: Integer): Boolean;
begin
  Result := TMXAbstractValue(FList[Idx]).GetBoolean;
end;

procedure TMXArrayValue.SetItemBoolean(Idx: Integer; AValue: Boolean);
begin
  TMXAbstractValue(FList[Idx]).Destroy;
  FList[Idx] := TMXBooleanValue.Create(AValue);
end;

function TMXArrayValue.GetItemString(Idx: Integer): UTF8String;
begin
  Result := TMXAbstractValue(FList[Idx]).GetString;
end;

procedure TMXArrayValue.SetItemString(Idx: Integer; const AValue: UTF8String);
begin
  TMXAbstractValue(FList[Idx]).Destroy;
  FList[Idx] := TMXStringValue.Create(AValue);
end;

function TMXArrayValue.GetItemDateTime(Idx: Integer): TDateTime;
begin
  Result := TMXAbstractValue(FList[Idx]).GetDateTime;
end;

procedure TMXArrayValue.SetItemDateTime(Idx: Integer; AValue: TDateTime);
begin
  TMXAbstractValue(FList[Idx]).Destroy;
  FList[Idx] := TMXDateTimeValue.Create(AValue);
end;

function TMXArrayValue.GetItemRecord(Idx: Integer): TMXRecordValue;
var Value: TMXAbstractValue;
begin
  Value := TMXAbstractValue(FList[Idx]);
  if Value.GetValueType <> mxRecordValue then
  begin
    Value.Destroy;
    Value := TMXRecordValue.Create;
    FList[Idx] := Value;
  end; // if
  Result := TMXRecordValue(Value);
end;

function TMXArrayValue.GetItemArray(Idx: Integer): TMXArrayValue;
var Value: TMXAbstractValue;
begin
  Value := TMXAbstractValue(FList[Idx]);
  if Value.GetValueType <> mxArrayValue then
  begin
    Value.Destroy;
    Value := TMXArrayValue.Create;
    FList[Idx] := Value;
  end; // if
  Result := TMXArrayValue(Value);
end;

function TMXArrayValue.GetItem(Idx: Integer): TMXAbstractValue;
begin
  Result := TMXAbstractValue(FList[Idx]);
end;

procedure TMXArrayValue.AddInteger(AValue: Integer);
var Value: TMXAbstractValue;
begin
  Value := TMXIntegerValue.Create(AValue);
  try
    FList.Add(Value);
  except
    Value.Free;
    raise;
  end;
end;

procedure TMXArrayValue.AddFloat(AValue: Extended);
var Value: TMXAbstractValue;
begin
  Value := TMXFloatValue.Create(AValue);
  try
    FList.Add(Value);
  except
    Value.Free;
    raise;
  end;
end;

procedure TMXArrayValue.AddBoolean(AValue: Boolean);
var Value: TMXAbstractValue;
begin
  Value := TMXBooleanValue.Create(AValue);
  try
    FList.Add(Value);
  except
    Value.Free;
    raise;
  end;
end;

procedure TMXArrayValue.AddString(const AValue: UTF8String);
var Value: TMXAbstractValue;
begin
  Value := TMXStringValue.Create(AValue);
  try
    FList.Add(Value);
  except
    Value.Free;
    raise;
  end;
end;

procedure TMXArrayValue.AddDateTime(AValue: TDateTime);
var Value: TMXAbstractValue;
begin
  Value := TMXDateTimeValue.Create(AValue);
  try
    FList.Add(Value);
  except
    Value.Free;
    raise;
  end;
end;

function TMXArrayValue.AddRecord: TMXRecordValue;
begin
  Result := TMXRecordValue.Create;
  try
    FList.Add(Result);
  except
    Result.Free;
    raise;
  end;
end;

function TMXArrayValue.AddArray: TMXArrayValue;
begin
  Result := TMXArrayValue.Create;
  try
    FList.Add(Result);
  except
    Result.Free;
    raise;
  end;
end;

procedure TMXArrayValue.Delete(Idx: Integer);
begin
  TMXAbstractValue(FList[Idx]).Destroy;
  FList.Delete(Idx);
end;

constructor TMXValue.Create;
begin
  FValue := nil;
end;

destructor TMXValue.Destroy;
begin
  if FValue <> nil then FValue.Destroy;
end;

function TMXValue.GetValueType: TMXValueType;
begin
  if FValue <> nil then Result := FValue.GetValueType
  else Result := mxNullValue;
end;

function TMXValue.GetInteger: Integer;
begin
  if FValue <> nil then Result := FValue.GetInteger
  else Result := 0;
end;

procedure TMXValue.SetInteger(AValue: Integer);
begin
  if FValue <> nil then FValue.Destroy;
  FValue := TMXIntegerValue.Create(AValue);
end;

function TMXValue.GetFloat: Extended;
begin
  if FValue <> nil then Result := FValue.GetFloat
  else Result := 0;
end;

procedure TMXValue.SetFloat(AValue: Extended);
begin
  if FValue <> nil then FValue.Destroy;
  FValue := TMXFloatValue.Create(AValue);
end;

function TMXValue.GetBoolean: Boolean;
begin
  if FValue <> nil then Result := FValue.GetBoolean
  else Result := false;
end;

procedure TMXValue.SetBoolean(AValue: Boolean);
begin
  if FValue <> nil then FValue.Destroy;
  FValue := TMXBooleanValue.Create(AValue);
end;

function TMXValue.GetString: UTF8String;
begin
  if FValue <> nil then Result := FValue.GetString
  else Result := '';
end;

procedure TMXValue.SetString(const AValue: UTF8String);
begin
  if FValue <> nil then FValue.Destroy;
  FValue := TMXStringValue.Create(AValue);
end;

function TMXValue.GetDateTime: TDateTime;
begin
  if FValue <> nil then Result := FValue.GetDateTime
  else Result := 0;
end;

procedure TMXValue.SetDateTime(AValue: TDateTime);
begin
  if FValue <> nil then FValue.Destroy;
  FValue := TMXDateTimeValue.Create(AValue);
end;

function TMXValue.GetRecord: TMXRecordValue;
begin
  if FValue <> nil then
  begin
    if FValue.GetValueType <> mxRecordValue then
    begin
      FValue.Destroy;
      FValue := TMXRecordValue.Create;
    end; // if
    Result := TMXRecordValue(FValue);
  end
  else
  begin
    Result := TMXRecordValue.Create;
    FValue := Result;
  end; // if
end;

function TMXValue.GetArray: TMXArrayValue;
begin
  if FValue <> nil then
  begin
    if FValue.GetValueType <> mxArrayValue then
    begin
      FValue.Destroy;
      FValue := TMXArrayValue.Create;
    end; // if
    Result := TMXArrayValue(FValue);
  end
  else
  begin
    Result := TMXArrayValue.Create;
    FValue := Result;
  end; // if
end;

procedure TMXValue.SetNull;
begin
  if FValue <> nil then
  begin
    FValue.Destroy;
    FValue := nil;
  end; // if
end;

end.
