unit mxwriter;

{$IFDEF FPC}
 {$MODE DELPHI}
{$ENDIF}
{$ASSERTIONS ON}

interface
uses SysUtils, Classes, mxvalues;

type
  TMXWriter = class
    private
      // поток куда нужно записать
      FStream: TStream;
      
      // нужно удалить нижележащий поток (FStream) при удалении TMXWriter
      FFreeOnDestroy: Boolean;
      
      // атрибуты
      FAttrs: TMXRecordValue;
      
      // тег открыт?
      FOpen: Boolean;
      
      // стек открытых тегов
      FStack: TStringList;
      
    protected
      
      // вывести текст без экранирования
      procedure Output(const Str: UTF8String);
      
      // записать атрибуты тега
      procedure WriteAttrs;
      
      // завешить открывающий тег
      procedure CompliteTag;
      
      // записать имя тега в стек
      procedure PushTag(const TagName: UTF8String);
      
      // извлечь имя тега из стека
      function PopTag: UTF8String;
      
    public
      constructor Create(AStream: TStream; FreeOnDestroy: Boolean = false);
      destructor Destroy; override;
      
      // открыть тег
      procedure Open(const TagName: UTF8String);
      
      // записать строку
      procedure Write(const Str: UTF8String; Escape: Boolean = true);
      
      // записать тег со сзначением
      procedure WriteTag(const TagName, TagValue: UTF8String);
      
      // закрыть тег
      procedure Close(const TagName: UTF8String);
      
      // записать все накопленые данные в поток
      procedure Flush;
      
      // атрибуты текущего тега
      property Attrs: TMXRecordValue read FAttrs;
  end;

function xml_escape(const str: UTF8String): UTF8String;

implementation

// TODO optimize
function xml_escape(const str: UTF8String): UTF8String;
var i, Len: Integer;
begin
  Len := Length(str);
  Result := '';
  for i := 1 to Len do
    case str[i] of
    '<': Result := Result + '&lt;';
    '>': Result := Result + '&gt;';
    '&': Result := Result + '&amp;';
    '"': Result := Result + '&quot;';
    else Result := Result + str[i];
    end;
end;

constructor TMXWriter.Create(AStream: TStream; FreeOnDestroy: Boolean);
begin
  FStream := AStream;
  FAttrs := TMXRecordValue.Create;
  FStack := TStringList.Create;
  FFreeOnDestroy := FreeOnDestroy;
  FOpen := false;
  Output('<?xml version="1.0"?>');
end;

destructor TMXWriter.Destroy;
begin
  FStack.Free;
  FAttrs.Free;
  if FFreeOnDestroy then FStream.Free;
end;

procedure TMXWriter.Output(const Str: UTF8String);
begin
  if Str <> '' then FStream.Write(Str[1], Length(Str));
end;

procedure TMXWriter.WriteAttrs;
var i, len: Integer;
begin
  len := FAttrs.Count;
  for i := 0 to len-1 do
  begin
    Output(' ' + FAttrs.Names[i] + '="' + xml_escape(FAttrs.Fields[i].AsString) + '"');
  end; // for i
end;

procedure TMXWriter.CompliteTag;
begin
  if FOpen then
  begin
    WriteAttrs;
    Output('>');
    FOpen := false;
  end; // if
end;

procedure TMXWriter.PushTag(const TagName: UTF8String);
begin
  FStack.Add(TagName);
end;

function TMXWriter.PopTag: UTF8String;
var len: Integer;
begin
  len := FStack.Count;
  assert(len > 0, 'stack is empty');
  Result := FStack[len-1];
  FStack.Delete(len-1);
end;

procedure TMXWriter.Open(const TagName: UTF8String);
begin
  CompliteTag;
  Output('<' + TagName);
  FAttrs.Clear;
  PushTag(TagName);
  FOpen := true;
end;

procedure TMXWriter.Write(const Str: UTF8String; Escape: Boolean);
begin
  CompliteTag;
  if Escape then Output(xml_escape(str))
  else Output(str);
end;

procedure TMXWriter.WriteTag(const TagName, TagValue: UTF8String);
begin
  Open(TagName);
  Write(TagValue);
  Close(TagName);
end;

procedure TMXWriter.Close(const TagName: UTF8String);
begin
  assert(PopTag = TagName, 'unexpected close tag');
  CompliteTag;
  if FOpen then
  begin
    WriteAttrs;
    Output(' />');
    FOpen := false;
  end
  else
  begin
    Output('</' + TagName + '>');
  end; // if
end;

procedure TMXWriter.Flush;
begin
  CompliteTag;
end;

end.
